Package: dslabs
Title: Data Science Labs
Version: 0.9.1
Description: Datasets and functions that can be used for data analysis practice, homework and projects in data science courses and workshops. 26 datasets are available for case studies in data visualization, statistical inference, modeling, linear regression, data wrangling and machine learning.
Authors@R: 
  c(person(given = c("Rafael", "A."), 
           family = "Irizarry", 
           role = c("aut", "cre"), 
           email = "rafael_irizarry@dfci.harvard.edu"),
    person(given = "Amy",
           family = "Gill",
           role = "aut"))
Depends: R (>= 4.1.0)
Imports: ggplot2
License: Artistic-2.0
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2025-12-01 19:56:57 UTC; rafa
Author: Rafael A. Irizarry [aut, cre],
  Amy Gill [aut]
Maintainer: Rafael A. Irizarry <rafael_irizarry@dfci.harvard.edu>
Repository: CRAN
Date/Publication: 2025-12-02 06:10:32 UTC
Built: R 4.6.0; ; 2026-04-26 06:43:19 UTC; unix

Package: dslabs
Title: Data Science Labs
Version: 0.9.1
Description: Datasets and functions that can be used for data analysis practice, homework and projects in data science courses and workshops. 26 datasets are available for case studies in data visualization, statistical inference, modeling, linear regression, data wrangling and machine learning.
Authors@R: 
  c(person(given = c("Rafael", "A."), 
           family = "Irizarry", 
           role = c("aut", "cre"), 
           email = "rafael_irizarry@dfci.harvard.edu"),
    person(given = "Amy",
           family = "Gill",
           role = "aut"))
Depends: R (>= 4.1.0)
Imports: ggplot2
License: Artistic-2.0
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2025-12-01 19:56:57 UTC; rafa
Author: Rafael A. Irizarry [aut, cre],
  Amy Gill [aut]
Maintainer: Rafael A. Irizarry <rafael_irizarry@dfci.harvard.edu>
Repository: CRAN
Date/Publication: 2025-12-02 06:10:32 UTC
Built: R 4.6.0; ; 2026-04-28 01:29:30 UTC; unix

# Generated by roxygen2: do not edit by hand

export(ds_theme_set)
export(fit_recommender_model)
export(read_mnist)
export(rfalling_object)
export(take_poll)
importFrom(stats,rnorm)
importFrom(stats,rt)
importFrom(utils,download.file)

library(tidyverse)
library(stringr)
### To create this RDA you need access to the file ProjectTycho_Level1_v1.0.0.csv
### Which is currently available only after opening an account here: https://www.tycho.pitt.edu/
### And downloading from Level 1 Data here: https://www.tycho.pitt.edu/data/#datasets
### Once you have this file set path to the path of the file:
path <- "/Users/rafa/myDocuments/teaching/data-science/labs/vaccines/data"

### Read in tables
filename <- file.path(path, "ProjectTycho_Level1_v1.0.0.csv")
the_diseases <- c("Hepatitis A", "Measles", "Mumps", "Pertussis", "Polio", "Rubella", "Smallpox")
### Tidy each table
us_contagious_diseases <- read_csv(filename, na = "\\N") %>%
  distinct() %>% ## removing duplicate rows
  mutate(disease = str_to_title(disease)) %>% 
  filter(disease %in% the_diseases) %>%
  mutate(loc = factor(str_to_title(loc))) %>%
  mutate(year = floor(epi_week/100)) %>%
  mutate(week = epi_week - year*100) %>%
  group_by(disease, loc, year) %>% 
  summarize(weeks_reporting = sum(!is.na(cases)), count = sum(cases, na.rm=TRUE)) %>%
  ungroup() %>%
  group_by(disease) %>%
  complete(loc, year, fill = list(weeks_reporting = 0, count = 0)) %>%
  rename(state = loc)

library("historydata")
### Now get the population data from historydaata package
## Function to interpolate population  between census years using a spline
M <- max(us_contagious_diseases$year)
us_state_populations_ext <- 
  purrr::map_df(levels(us_contagious_diseases$state), function(x){
    dat <- filter(us_state_populations, state == x)
    data.frame(state = I(x),
             year = seq(min(dat$year), M),
             population = round(exp(spline(dat$year, log(dat$population), method = "natural",
                                                        xout = seq(min(dat$year), M))$y)))
}) %>% 
  mutate(state = factor(state))

## Add population data to disease tidy table
us_contagious_diseases <- left_join(us_contagious_diseases, us_state_populations_ext, by = c("state","year"))

### turn into data frame and make some columns into factors
us_contagious_diseases <- data.frame(us_contagious_diseases)
us_contagious_diseases$disease <- factor(us_contagious_diseases$disease)

save(us_contagious_diseases, file = "data/us_contagious_diseases.rda", compress="xz", version = 2)

library(tidyverse)

tab <- read_csv("https://docs.google.com/spreadsheet/pub?key=phAwcNAVuyj0TAlJeCEzcGQ&output=csv",  na = c("", "NA", "-", "."))
countries <- c("South Korea","Germany")
names(tab)[1] <- "country"
tab <- tab %>% filter(country %in% countries) %>% select(country,`1960`:`2015`)
write.csv(tab, "inst/extdata/fertility-two-countries-example.csv", row.names = FALSE)

tab1 <- tab
tab2 <- read_csv("http://docs.google.com/spreadsheet/pub?key=phAwcNAVuyj2tPLxKvvnNPA&output=csv")
countries <- c("South Korea","Germany")
names(tab2)[1] <- "country"
tab2 <- tab2 %>% filter(country %in% countries) %>% select(country,`1960`:`2015`)

colnames(tab1)[-1] <- paste(colnames(tab1)[-1], "fertility", sep="_")
colnames(tab2)[-1] <- paste(colnames(tab2)[-1], "life_expectancy", sep="_")

tab3 <- full_join(tab1, tab2, by="country")
tab3 <- select(tab3, c("country", sort(names(tab3)[-1])))
write.csv(tab3, "inst/extdata/life-expectancy-and-fertility-two-countries-example.csv", row.names = FALSE)

# import stars data for HR diagram 
library(tidyverse)
stars <- read.table("inst/extdata/HRlist2.txt")
names(stars) <- c("star", "magnitude", "temp", "type")
stars$type <- str_remove(stars$type, "[^A-Z]+")
save(stars, file = "data/stars.rda", compress = "xz")

## Code written by Amy Gill

##Set working directory to package main directory
library(dplyr)
library(rvest)
library(readr)
library(stringr)
library(lubridate)

## get data from wikipedia
url <- "https://en.wikipedia.org/w/index.php?title=Opinion_polling_for_the_United_Kingdom_European_Union_membership_referendum&oldid=896735054"
h <- read_html(url)
brexit_polls <- h %>% html_nodes("table") %>%
  html_table(fill=TRUE) %>% .[[7]] 

## change column names
brexit_polls <- brexit_polls[-1,]
names(brexit_polls) <- c("last_date", "remain", "leave", "undecided", "lead", "samplesize", "pollster", "poll_type", "notes")

## rows with noe data and not including the ninth column: the notes column 
brexit_polls <- brexit_polls[-which(!grepl("%", brexit_polls$remain)), 1:8]

## getting start and end dates
dates <- brexit_polls$last_date
end <- str_extract(dates, "\\d+\\s[a-zA-Z]+")
start_day <- str_extract(dates, "^\\d{1,2}")
months <- str_extract_all(dates, "[a-zA-Z]{3,4}", simplify = TRUE)
end <- paste(end, "2016")
enddate <- dmy(end)
startdate <- ymd(paste(year(enddate), months[,1], start_day))
brexit_polls$enddate <- enddate
brexit_polls$startdate <- startdate

#remove final results line
brexit_polls <- brexit_polls[-1,]

## remove lead columns which is redundant
brexit_polls <- select(brexit_polls, -c("lead", "last_date"))

## convert poll percentages to proportions
brexit_polls$remain <- parse_number(brexit_polls$remain) / 100
brexit_polls$leave <- parse_number(brexit_polls$leave) / 100
brexit_polls$undecided <- brexit_polls$undecided %>% 
  str_replace_all("N/A", "0") %>%
  parse_number %>% 
  "/"(100)

## conver pretty numbers to actual numbers and take out poll with no sample size
brexit_polls <- brexit_polls %>% filter(samplesize != "N/A") %>%
  mutate(samplesize = parse_number(samplesize))

## conver pollsters to factors
brexit_polls$pollster <- factor(brexit_polls$pollster)
brexit_polls$poll_type <- factor(brexit_polls$poll_type)

## compute the spread
brexit_polls <- brexit_polls %>%
  mutate(spread = remain - leave)

brexit_polls <- brexit_polls[, c("startdate", "enddate", "pollster", "poll_type", "samplesize", "remain", "leave", "undecided", "spread")]


save(brexit_polls, file="data/brexit_polls.rda", compress = "xz")

library(tidyverse)

# import global tempearture anomaly data from NOAA
url <- "https://www.ncdc.noaa.gov/cag/global/time-series/globe/land_ocean/ytd/12/1880-2019.csv"
temps <- read_csv(url, skip = 4)
colnames(temps) = c("year", "temp_anomaly")

# import land temperature anomaly data from NOAA
url <- "https://www.ncdc.noaa.gov/cag/global/time-series/globe/land/ytd/12/1880-2019.csv"
land_temps <- read_csv(url, skip = 4)
colnames(land_temps) <- c("year", "land_anomaly")

# import ocean temperature anomaly data from NOAA
url <- "https://www.ncdc.noaa.gov/cag/global/time-series/globe/ocean/ytd/12/1880-2019.csv"
ocean_temps <- read_csv(url, skip = 4)
colnames(ocean_temps) <- c("year", "ocean_anomaly")

# combine temperature anomaly data
temps <- left_join(temps, land_temps, by = "year")
temps <- left_join(temps, ocean_temps, by = "year")

# import CO2 emissions data from CDIAC - selected columns were manually removed before import to avoid parsing failures
# original data: https://cdiac.ess-dive.lbl.gov/trends/emis/tre_glob_2014.html
Carbon_Emissions <- read_csv("inst/extdata/carbon_emissions.csv")

# rename columns
names(Carbon_Emissions) <- c("year","carbon_emissions")

# merge temps, Carbon_Emissions
temp_carbon <- full_join(temps, Carbon_Emissions, by = "year")

# remove temporary objects
rm(Carbon_Emissions, temps, land_temps, ocean_temps, url)

# make data frame
temp_carbon <- as.data.frame(temp_carbon)

# save to rda
save(temp_carbon, file = "data/temp_carbon.rda", compress = "xz")

##2012 results
library(rvest)
library(readr)
library(tidyverse)
library(stringr)

url <- "https://en.wikipedia.org/w/index.php?title=2012_United_States_presidential_election&oldid=1264588444"
tmp <- read_html(url) |> html_table() |> _[[25]] |> _[-c(1,58), c(1, 20, seq(2, 11, 3), c(4, 7))]
x <- tmp |> setNames(c("state",  "total", 
                       tolower(str_replace(names(tmp[3:6]), "^([^A-Z]*[A-Z][a-z]*[^A-Z]*){2}[A-Z][a-z]*", "\\1")), 
                       paste0("EV",1:2))) |>
  mutate(across(-state, ~ if_else(. == "–", "0", .)))
x$electoral_votes <- apply(x[,c("EV1","EV2")], 1, function(x) sum(as.numeric(x), na.rm = TRUE))
x <- select(x,-c("EV1","EV2"))
x <- x |> mutate(across(-c(state, electoral_votes), parse_number)) |>
  mutate(state = str_replace(x$state,"Tooltip.*$|†|\\[\\d+\\]|District of Columbia", "")) 
x$sum <- rowSums(x[,3:5])
x <- x |> mutate(total = if_else(sum > total, sum, total)) |>
  select(-sum) |>  
  mutate(across(-c(state, total, electoral_votes), ~ . / total * 100)) |>
  select(-total) |>
  arrange(desc(electoral_votes)) |>
  relocate(electoral_votes, .after = state) |>
  mutate(state = str_replace_all(state, c("ME-" = "Maine CD-", "NE-" = "Nebraska CD-")))



results_us_election_2012 <- as.data.frame(x)
save(results_us_election_2012, file = "data/results_us_election_2012.rda", compress = "xz")

library(tidyverse)
mice_weights <- read.csv("/Users/rafa/myDocuments/teaching/data-science/labs/mouse-weights/svenson850_phenotypes_2021.csv.gz") |>
  filter(gen != 5 & !is.na(bw.19)) |> ## Gen 5 had no animals on high-fat diet
  select(bw.19,  bmd2, perc.fat2, sex, diet, gen, litter) |> 
  rename(body_weight = bw.19, bone_density = bmd2, percent_fat = perc.fat2 ) |>
  mutate(sex = factor(sex), diet = factor(diet), gen = as.factor(gen),
         litter = factor(litter)) |>
  filter(!is.na(body_weight))  |>
  arrange(diet, sex, gen, litter) 

save(mice_weights, file="data/mice_weigths.rda", compress = "xz", version = 2)

## Create a spreadsheet to practice wrangling with different encoding and locale
library(lubridate)
set.seed(2023 - 9 - 10)
original_time <- Sys.getlocale(category = "LC_TIME")
olocale <- Sys.getlocale()
Sys.setlocale("LC_TIME", "es_ES.UTF-8")
nombre <- c("Plácido", "João", "Beyoncé", "López", "Thalía", "Blümchen", "Ñengo") 
df <- data.frame(
  nombre = nombre,
  f.n. = format(c(make_date(1941, 1, 24),
                 make_date(1931, 6, 10),
                 make_date(1981, 9, 4),
                 make_date(1969, 7, 24), 
                 make_date(1971, 8, 26), 
                 make_date(1980, 4, 20),
                 make_date(1981, 12, 15)), "%d de %B de %Y"),
  estampa = make_datetime(
    year = year(today()),
    month = month(today()),
    day = day(today()),
    hour = sample(17:23, length(nombre), replace = TRUE),
    min = sample(0:59, length(nombre), replace = TRUE),
    sec = sample(0:59, length(nombre), replace = TRUE),
    tz = "America/New_York") |> with_tz(tz = "UTC"),
  puntuación = format(round(runif(length(nombre),80,100),1), decimal.mark = ",")
)


df <- df[order(df$nombre),]
# Write the data frame to CSV in ISO-8859-1 encoding
write.csv(df, file = "inst/extdata/calificaciones.csv", fileEncoding = "ISO-8859-1",
          row.names = FALSE)

## test
Sys.setlocale("LC_TIME", original_time)

library(readr)
try({x <- read_csv("inst/extdata/calificaciones.csv")})
x <- read_csv("inst/extdata/calificaciones.csv", locale = locale(encoding = "ISO-8859-1", decimal_mark = ","))
parse_date(x$f.n., format = "%d de %B de %Y", locale = locale(date_names = "es"))
with_tz(x$estampa, tz =  Sys.timezone())

library(tidyverse)
library(readxl)

nyc_regents_scores <- read_excel("inst/extdata/2010_bigfive_regents.xls") %>% as.data.frame
nyc_regents_scores <- nyc_regents_scores[-c(1:2),]
names(nyc_regents_scores) <- gsub("\\.", "", gsub(" ", "_", tolower(names(nyc_regents_scores))))
rownames(nyc_regents_scores) <- NULL
names(nyc_regents_scores)[1] <- "score"
nyc_regents_scores$score[nyc_regents_scores$score == 999] <- NA
nyc_regents_scores$score <- as.numeric(nyc_regents_scores$score)
nyc_regents_scores

save(nyc_regents_scores, file = "data/nyc_regents_scores.rda", compress="xz")

##Set working directory to package main directory
library(dplyr)
library(rvest)
library(readr)
library(stringr)

### US murders 2010

url <- "https://en.wikipedia.org/w/index.php?title=Gun_violence_in_the_United_States_by_state&direction=prev&oldid=810166167"
h <- read_html(url)
us_murders <- h %>% html_nodes("table") %>%  
  html_table %>% .[[1]] %>% 
  tibble %>% setNames(c("state", "population", "population_density", 
                        "murders", "gun_murders", "gun_ownership", "murder_rate", "gun_murder_rate"))

to_parse_number <- names(us_murders)[-1]
us_murders %>% mutate_at(to_parse_number, parse_number)

columns_to_add <- data_frame(state = c(state.name, "District of Columbia"),
                             abb = c(state.abb,"DC"),
                             region = factor( c(as.character(state.region), "South"), 
                                              levels = levels(state.region)))

us_murders <- full_join(us_murders, columns_to_add, by = "state")

murders <- with(us_murders,
                data.frame(state = state,
                      abb = abb,
                      region = region,
                      population = parse_number(population),
                      total = parse_number(us_murders$gun_murders),
                      stringsAsFactors = FALSE))

save(murders, file = "data/murders.rda")
                

set.seed(1)
### heights in feet
outlier_example <- round(rnorm(500, 69/12, 3/12), 2)
## we pick a value that will be about 180 centimeters
o <- 219
outlier_example[o] <- round(outlier_example[o]*12*2.54/5)*5 ##mistake made in o-th measurement, centimeters, rounded to closest 5, reported
save(outlier_example, file = "data/outlier_example.rda")

#devtools::install_github("genomicsclass/tissuesGeneExpression")
##rafalib::install_bioc("hgu133plus2.db")
library(hgu133plus2.db)
library(tissuesGeneExpression)

data(tissuesGeneExpression)

map <- mapIds(hgu133plus2.db, 
              keytype="PROBEID", 
              column="SYMBOL",
              keys=rownames(e),
              mutliVals = first)

o <- order(tissue)
tissue <- tissue[o]
e <- e[,o]
rownames(e) <- map
colnames(e) <- paste(tissue, 
                     unlist(
                       sapply(table(tissue), function(n) 1:n)), 
                     sep="_")
## pick featuers with a gene name and make sure they are unique
ind <- which(!is.na(map) & !duplicated(map))

set.seed(1994)
e <- e[sample(ind, 500), ]

tissue_gene_expression <- list(x = t(e), y = factor(tissue))

save(tissue_gene_expression, 
     file="data/tissue_gene_expression.rda",
     compress = "xz")

### test
if(FALSE){
  library(caret)
  ind <- which(matrixStats::colSds(tissue_gene_expression$x) > 0.5)
  tissue_gene_expression$x <- with(tissue_gene_expression,
                                   sweep(x, 1, rowMeans(x)))
  fit <- train(tissue_gene_expression$x[,ind],
               tissue_gene_expression$y, method = "knn",
               tuneGrid = data.frame(k=seq(1,7,2)))
  plot(fit)
}

##Set working directory to package main directory
library(RCurl)
library(tidyverse)
library(stringr)
library(lubridate)

tc <- getForm("https://docs.google.com/spreadsheet/pub", 
             hl ="en_US", key = "0ApvpBbD8HP4mdDlRUi1vdTlBQ3Rub2dJSUNVUDlDdVE", 
             output = "csv", 
             .opts = list(followlocation = TRUE, verbose = TRUE, ssl.verifypeer = FALSE)) 
reported_heights_1 <- read.csv(textConnection(tc), stringsAsFactors = FALSE)[,c(1,3,2)]
reported_heights_2 <- read.csv("https://raw.githubusercontent.com/datasciencelabs/data/master/bio260-heights.csv",
                              stringsAsFactors = FALSE)

names(reported_heights_1) <- names(reported_heights_2) <- c("time_stamp", "sex", "height")

reported_heights_1  <- reported_heights_1 %>% mutate(time_stamp = mdy_hms(time_stamp))
reported_heights_2  <- reported_heights_2 %>% mutate(time_stamp = mdy_hms(time_stamp))


reported_heights <- bind_rows(reported_heights_1,
                          reported_heights_2)

## to help keep reproducible, remove entries from after the latest data
## from last time we saved the rda: "2017-10-14 14:07:51 UTC"

reported_heights <- filter(reported_heights, sex%in%c("Male","Female")) %>%
  filter(time_stamp <= "2017-10-14 14:07:51 UTC")  %>% arrange(time_stamp) %>%
  mutate(time_stamp = as.character(time_stamp))

save(reported_heights, file = "data/reported_heights.rda")
              
### Now clean it up
convert_format <- function(s){
  s %>%
    str_replace("feet|foot|ft", "'") %>% #convert feet symbols to '
    str_replace_all("inches|in|''|\"|cm|and", "") %>%  #remove inches and other symbols
    str_replace("^([4-7])\\s*[,\\.\\s+]\\s*(\\d*)$", "\\1'\\2") %>% #change x.y, x,y x y
    str_replace("^([56])'?$", "\\1'0") %>% #add 0 when missing
    str_replace("^([12])\\s*,\\s*(\\d*)$", "\\1\\.\\2") %>% #change european decimal
    str_trim() #remove extra space
}

words_to_numbers <- function(s){
  s %>% 
    str_to_lower() %>%  
    str_replace_all("zero", "0") %>%
    str_replace_all("one", "1") %>%
    str_replace_all("two", "2") %>%
    str_replace_all("three", "3") %>%
    str_replace_all("four", "4") %>%
    str_replace_all("five", "5") %>%
    str_replace_all("six", "6") %>%
    str_replace_all("seven", "7") %>%
    str_replace_all("eight", "8") %>%
    str_replace_all("nine", "9") %>%
    str_replace_all("ten", "10") %>%
    str_replace_all("eleven", "11")
}

## Now convert all heights
pattern <- "^([4-7])\\s*'\\s*(\\d+\\.?\\d*)$"

smallest <- 50
tallest <- 84
new_heights <- reported_heights %>% 
  mutate(original = height, 
         height = words_to_numbers(height) %>% convert_format()) %>%
  extract(height, c("feet", "inches"), regex = pattern, remove = FALSE) %>% 
  mutate_at(c("height", "feet", "inches"), as.numeric) %>%
  mutate(guess = 12*feet + inches) %>%
  mutate(height = case_when(
    !is.na(height) & between(height, smallest, tallest) ~ height, #inches 
    !is.na(height) & between(height/2.54, smallest, tallest) ~ height/2.54, #centimeters
    !is.na(height) & between(height*100/2.54, smallest, tallest) ~ height*100/2.54, #meters
    !is.na(guess) & inches < 12 & between(guess, smallest, tallest) ~ guess, #feet'inches
    TRUE ~ as.numeric(NA))) %>%
  select(-guess)

heights <- data.frame(sex = reported_heights$sex, height = new_heights$height)
heights <- filter(heights, !is.na(height))

## balance it between male and female
rownames(heights) <- NULL

save(heights, file = "data/heights.rda")

## download data (takes a few minutes)
library(tidyverse)
url <- "http://files.grouplens.org/datasets/movielens/ml-latest-small.zip"
td <- tempdir()
print(td)
tf <- tempfile(tmpdir=td, fileext=".zip")
download.file(url, tf)

# to see files:
#unzip(tf, list = TRUE)

## read the data
movielens <- read_csv(unz(tf, "ml-latest-small/ratings.csv"))
movielens_titles <- read_csv(unz(tf, "ml-latest-small/movies.csv"))

## separate year from title and keep only recent movies
movielens_titles <- movielens_titles %>% 
  extract(title, c("title", "year"), regex = "(.*)\\s\\((\\d+)\\)", convert = TRUE) 

movielens <- right_join(movielens_titles, movielens, by = "movieId")

movielens <- as.data.frame(movielens)
movielens$genres <- factor(movielens$genres)
## check
#movielens  %>% group_by(movieId) %>% summarize(avg = mean(rating), n=n(), title= title[1]) %>% arrange(desc(avg))
#movielens %>%  group_by(movieId) %>% summarize(avg = mean(rating), n=n(), title= title[1]) %>% filter(n>100) %>% arrange(desc(avg))
#movielens %>%  group_by(movieId) %>% summarize(avg = mean(rating), n=n(), title= title[1]) %>% filter(n>10) %>% arrange(avg)

save(movielens, file = "data/movielens.rda", compress="xz")

unlink(td, recursive = TRUE, force=TRUE)

olive <- read.csv("inst/extdata/olive.csv")
olive$Area <- factor(olive[,1])
olive <- olive[,-1]
olive[ , -c(1,2)] <- olive[ , -c(1,2)]/100
region_names <- c("Southern Italy","Sardinia","Northern Italy")
olive$Region <- factor(region_names[olive$Region])
names(olive) <- tolower(names(olive))
save(olive, file="data/olive.rda", compress="xz")

library(readr)
library(tidyverse)
library(stringr)
library(rvest)

##polls
url <- "http://projects.fivethirtyeight.com/general-model/president_general_polls_2016.csv"
polls_us_election_2016 <- read_csv(url)
polls_us_election_2016 <- filter(polls_us_election_2016, type == "polls-plus")
polls_us_election_2016$startdate <- as.Date(polls_us_election_2016$startdate,"%m/%d/%Y") ##turn enddate into date
polls_us_election_2016$enddate <- as.Date(polls_us_election_2016$enddate,"%m/%d/%Y") 
polls_us_election_2016 <- select(polls_us_election_2016, state, startdate, enddate, pollster, grade, samplesize, population, rawpoll_clinton, rawpoll_trump, rawpoll_johnson, rawpoll_mcmullin, adjpoll_clinton, adjpoll_trump, adjpoll_johnson, adjpoll_mcmullin)
polls_us_election_2016 <- as.data.frame(polls_us_election_2016)
polls_us_election_2016 <- mutate(polls_us_election_2016, 
                                 state = factor(state), 
                                 pollster = factor(pollster), 
                                 grade = factor(grade, levels = c("D","C-","C","C+","B-","B","B+","A-","A","A+")))

##2016 results
url <- "https://en.wikipedia.org/w/index.php?title=2016_United_States_presidential_election&oldid=1265623400"
tmp <- read_html(url) |> html_nodes("table") |> _[[43]] |> html_table()  |>
  _[-c(1,58,59), c(1, 23, seq(2, 17, 3), seq(4, 19, 3))]
x <- tmp |> setNames(c("state",  "total", tolower(str_remove(names(tmp[3:8]), "/.*")),
                       paste0("EV",1:6))) |>
  mutate(across(-state, ~ if_else(. == "–", "0", .)))
x$electoral_votes <- select(x, contains("EV")) |> apply(1, function(x) sum(as.numeric(x), na.rm = TRUE))
x <- select(x, -contains("EV"))
x <- x |> mutate(across(-c(state, electoral_votes), parse_number)) |>
  mutate(state = str_trim(str_replace(x$state,"Tooltip.*$|†|\\[\\d+\\]", ""))) 
x$sum <- select(x, -c(state, total, electoral_votes)) |> rowSums()
x <- x |> mutate(total = if_else(sum > total, sum, total)) |>
  select(-sum) |>  
  mutate(across(-c(state, total, electoral_votes), ~ . / total * 100)) |>
  select(-total) |>
  arrange(desc(electoral_votes)) |>
  relocate(electoral_votes, .after = state) |>
  mutate(state = str_replace_all(state, c("ME-" = "Maine CD-", "NE-" = "Nebraska CD-")))

## we reorder for illustrate how join works
results_us_election_2016 <- as.data.frame(x) %>% arrange(desc(electoral_votes))  
save(polls_us_election_2016, results_us_election_2016, file = "data/polls_us_election_2016.rda", compress = "xz")

library(pdftools)
library(tidyverse)
library(lubridate)
library(purrr)
library(pdftools)

fn <- system.file("extdata", "RD-Mortality-Report_2015-18-180531.pdf", package = "dslabs")
x <- str_split(pdf_text(fn), "\n")
pr_death_counts <- map_df(x, function(s){
  s <- str_trim(s)
  header_index <- str_which(s, "2015")[1]
  tmp <- str_split(s[header_index], "\\s+", simplify = TRUE)
  month <- tmp[1]
  header <- tmp[-1]
  tail_index  <- str_which(s, "Total")
  n <- str_count(s, "\\d+")
  out <- c(1:header_index, which(n <= 3), 
           which(n >= 28 & n <= 31), tail_index:length(s))
  
  if (month == "FEB") {
    feb29 <- s[str_detect(s, "^29\\s+")] |> str_remove("29\\s+") |> parse_number()
  }
  s <- s[-out] |>  
    str_remove_all("[^\\d\\s]") |> ## remove things that are not digits or space
    str_trim() |> 
    str_split_fixed("\\s+", n = 6)  ## split by any space
  
  s <- s[,1:5]
  colnames(s) <- c("day", header)
  s <- s |> as_tibble(validate = FALSE) |> 
    mutate(month = month, day = as.numeric(day)) |>
    pivot_longer(-c(day, month), names_to = "year", values_to = "deaths") |>
    mutate(deaths = as.integer(deaths), month = str_to_title(month)) |>
    mutate(month = if_else(month == "Ago", "Aug", month)) |>
    mutate(month = match(month, month.abb)) |>
    mutate(date = make_date(year, month, day)) |>
    select(date, deaths) 
  if (month == "FEB") {
    s <- bind_rows(s, data.frame(date = make_date(2016, 2, 29), deaths = feb29)) 
  }
  return(s)
}) |> arrange(date) |> filter(date <= make_date(2018, 4, 15)) |> as.data.frame()

save(pr_death_counts, file = "data/pr-death-counts.rda", compress = "xz", version = 2)
  

library(tidyverse)

# import Mauna Loa annual mean CO2 concentrations from https://www.esrl.noaa.gov/gmd/ccgg/trends/data.html
url <- "ftp://aftp.cmdl.noaa.gov/products/trends/co2/co2_annmean_mlo.txt"
co2_mauna_loa <- read_table(url, skip = 56) %>%    # skip documentation
    select(year, co2 = mean) %>%    # keep annual mean CO2
    mutate(source = "Mauna Loa")    # label as Mauna Loa measurements

# import Antarctic Ice Cores Revised 800KYr CO2 Data: Bereiter et al. 2015: https://www.ncdc.noaa.gov/paleo-search/study/17975
url <- "https://www1.ncdc.noaa.gov/pub/data/paleo/icecore/antarctica/antarctica2015co2composite.txt"
co2_ice_cores <- read_tsv(url, skip = 137) %>%    # skip documentation
    mutate(year = round(-age_gas_calBP + 1950),    # convert year from calBP to CE
           source = "Ice Cores") %>%    # label as ice core measurements
    select(year, co2 = co2_ppm, source)    # keep CO2 ppm

# keep every 3rd measurement (reduce size of dataset)
rows <- seq(1, nrow(co2_ice_cores), 3)
co2_ice_cores <- co2_ice_cores[rows,]

# combine Mauna Loa and ice core CO2 measurements
historic_co2 <- rbind(co2_mauna_loa, co2_ice_cores)

# remove temporary objects
rm(co2_mauna_loa, co2_ice_cores)

# save as rda
save(historic_co2, file = "data/historic_co2.rda", compress = "xz")

library(tidyverse)
library(tidyr)
library(stringr)
library(readr)

## Download the table
library("pdftools")
temp_file <- tempfile()
url <- "http://www.pnas.org/content/suppl/2015/09/16/1510159112.DCSupplemental/pnas.201510159SI.pdf"
download.file(url, temp_file)
txt <- pdf_text(temp_file)
file.remove(temp_file)

raw_data_research_funding_rates <- txt[2]

save(raw_data_research_funding_rates, file="data/raw_data_research_funding_rates.rda", compress="xz")

## Get the names
tab <- str_split(raw_data_research_funding_rates, "\n")[[1]]
the_names_1 <- tab[3]
the_names_2 <- tab[4]

the_names_1 <- the_names_1 %>%
  str_trim() %>%
  str_replace_all(",\\s[n|%]", "") %>%
  str_split("\\s{2,}", simplify = TRUE)

the_names_2 <- the_names_2 %>%
  str_trim() %>%
  str_split("\\s+", simplify = TRUE)

tmp_names <- str_c(rep(the_names_1, each = 3), the_names_2[-1], sep = "_")
the_names <- c(the_names_2[1], tmp_names) %>%
  str_to_lower() %>%
  str_replace_all("\\s", "_")

## Create the table
research_funding_rates <- tab[6:14] %>%
  str_trim %>%
  str_split("\\s{2,}", simplify = TRUE) %>%
  data.frame(stringsAsFactors = FALSE) %>%
  setNames(the_names) %>%
  mutate_at(-1, parse_number)

save(research_funding_rates, file="data/research_funding_rates.rda", compress="xz")

library(dplyr)
library(readr)
library(readxl)
library(tidyr)
library(countrycode)


## Note that the location of these files change
## You might have to go here https://www.gapminder.org/data/ and find the new links
urls <- c("https://spreadsheets.google.com/pub?key=phAwcNAVuyj0NpF2PTov2Cw&output=csv",
          "https://spreadsheets.google.com/pub?key=phAwcNAVuyj2tPLxKvvnNPA&output=csv",
          "https://spreadsheets.google.com/pub?key=phAwcNAVuyj0TAlJeCEzcGQ&output=csv",
          "https://spreadsheets.google.com/pub?key=phAwcNAVuyj0XOoBL_n5tAQ&output=csv",
          "http://spreadsheets.google.com/pub?key=pyj6tScZqmEfI4sLVvEQtHw&output=csv")


names(urls) <- c("infant_mortality", "life_expectancy",
                 "fertility","population","gdp")

tables <- lapply(urls, read_csv, na = c("", "NA", "-", "."))

##Tidy data
##Function to make column name for key uniform and
##change for wide to long format
fix_table <- function(tab, val_name){
  names(tab)[1] <- "country"
  tab <- filter(tab, !is.na(country))
  tab <- gather(tab, key=year, value=y, -country, convert = TRUE)
  names(tab)[which(names(tab)=="y")] <- val_name
  return(tab)
}
##Fix each table
for(i in seq_along(tables) ) tables[[i]] <-fix_table(tables[[i]], names(tables)[i])

##Join into one large table
gapminder <- Reduce(full_join, tables)
gapminder <- filter(gapminder, year>=1960)

### remove countries that dont have the data needed for the main figures
out <- gapminder %>% group_by(country) %>% summarize_at(c("infant_mortality","life_expectancy","fertility","gdp"), function(x) all(is.na(x))) %>% 
  filter(life_expectancy | fertility | gdp) %>% .$country
out <- c(out, "Channel Islands") ##removed because no information on continent or region. 
gapminder <- filter(gapminder, !country%in%out)

rm(tables); gc(verbose = FALSE)

## Add continent and region
gapminder <- mutate(gapminder, 
                    continent = factor(countrycode(country, "country.name", "continent")),
                    region = factor(countrycode(country, "country.name", "region"))) %>%
  mutate(country = factor(country))

gapminder <- as.data.frame(gapminder)

oecd <- c("Australia","Austria","Belgium","Canada","Chile","Country","Czech Republic","Denmark","Estonia","Finland","France","Germany","Greece","Hungary","Iceland","Ireland","Israel","Italy","Japan","Korea","Luxembourg","Mexico","Netherlands","New Zealand","Norway","Poland","Portugal","Slovak Republic","Slovenia","Spain","Sweden","Switzerland","Turkey","United Kingdom","United States")
opec <- c("Algeria","Angola","Ecuador","Iran","Iraq","Kuwait","Libya","Nigeria","Qatar","Saudi Arabia","United Arab Emirates","Venezuela")

save(gapminder, oecd, opec, file="data/gapminder.rda", compress="xz")

# import SSA period life expectancy tables
library(tidyverse)
death_prob <- read.csv("inst/extdata/ssa-death-probability.csv")
death_prob <- select(death_prob, "Age", "Sex", "DeathProb") %>%
  setNames(tolower(names(.))) %>%
  rename(prob = deathprob)

save(death_prob, file = "data/death_prob.rda", compress = "xz")

library(tidyverse)
library(rvest)

url <- "https://en.wikipedia.org/wiki/Simpson%27s_paradox#Berkeley_gender_bias_case"
h <- read_html(url)

admissions <- h %>% 
  html_nodes("table") %>% 
  .[[2]] %>% 
  html_table(fill=TRUE)  %>%
  setNames(tolower(c("major", paste(names(.)[-1], .[1,-1], sep="_")))) %>%
  slice(2:n()) %>%
  gather(key, value, -major) %>%
  separate(key, c("gender", "key"), "_")  %>%
  spread(key, value) %>% 
  arrange(gender) %>%
  mutate_at(c("admitted","applicants"), funs(parse_number)) %>%
  data.frame(., stringsAsFactors = FALSE)

save(admissions, file="data/admissions.rda", compress="xz")

library(tidyverse)
library(lubridate)
library(rvest)
url <- "https://web.archive.org/web/20160309010059/http://www.pollster.com/08USPresGEMvO-2.html"

polls_2008 <- read_html(url) %>%  
  html_node("table") %>%
  html_table(header=TRUE) %>%
  separate(col=Dates, into=c("start_date","end_date"), sep="-", fill="right") %>% 
  mutate(end_date = ifelse(is.na(end_date), start_date, end_date)) %>% 
  separate(start_date, c("smonth", "sday", "syear"), sep = "/",  convert = TRUE, fill = "right")%>% 
  mutate(end_date = ifelse(str_count(end_date, "/") == 1, paste(smonth, end_date, sep = "/"), end_date)) %>% 
  mutate(end_date = mdy(end_date))  %>% mutate(syear = ifelse(is.na(syear), year(end_date), syear + 2000)) %>% 
  unite(start_date, smonth, sday, syear)  %>% 
  mutate(start_date = mdy(start_date)) %>% 
  separate(`N/Pop`, into=c("N","population_type"), sep="\ ", convert=TRUE, fill="left") %>% 
  mutate(Obama = as.numeric(Obama)/100, 
         McCain=as.numeric(McCain)/100,
         Barr=as.numeric(na_if(Barr,"-"))/100,
         Nader=as.numeric(na_if(Nader,"-"))/100,
         Other=as.numeric(na_if(Other,"-"))/100,
         Undecided=as.numeric(na_if(Undecided,"-"))/100,
         Margin = Obama - McCain,
         Day = as.numeric(start_date - mdy("11/04/2008"))) %>%
  filter(start_date >= "2008-06-01") %>%
  group_by(day=Day)  %>% 
  summarize(margin=mean(Margin))
save(polls_2008, file = "data/polls_2008.rda", compress="xz")

library(caret)
source("R/read_mnist.R")
if (!exists("mnist")) mnist <- read_mnist()
set.seed(3456)
index_127 <- sample(which(mnist$train$labels %in% c(1,2,7)), 2000)
y <- mnist$train$labels[index_127] 
x <- mnist$train$images[index_127,]
index_train <- createDataPartition(y, p = 0.8, list = FALSE)
## get the quadrants
row_column <- expand.grid(row = 1:28, col = 1:28) 
upper_left_ind <- which(row_column$col <= 14 & row_column$row <= 14)
lower_right_ind <- which(row_column$col > 14 & row_column$row > 14)
## binarize the values. Above 200 is ink, below is no ink
x <- x > 200 
## proportion of pixels in lower right quadrant
x <- cbind(rowSums(x[ ,upper_left_ind])/rowSums(x), 
           rowSums(x[ ,lower_right_ind])/rowSums(x)) 
##save data
train_set <- data.frame(y = factor(y[index_train]),
                        x_1 = x[index_train,1], x_2 = x[index_train,2])
test_set <- data.frame(y = factor(y[-index_train]),
                       x_1 = x[-index_train,1], x_2 = x[-index_train,2])

mnist_127 <- list(
  train = data.frame(y = factor(y[index_train]),
                     x_1 = x[index_train,1],
                     x_2 = x[index_train,2]),
  test = data.frame(y = factor(y[-index_train]),
                    x_1 = x[-index_train,1],
                    x_2 = x[-index_train,2]))

save(mnist_127, file = "data/mnist_127.rda", compress = "xz", version = 2)

# import Wisconsin Diagnostic Breast Cancer Dataset from UCI Machine Learning Repository
# citation: W.N. Street et al., 1993

library(tidyverse)

# import data
url <- "http://mlr.cs.umass.edu/ml/machine-learning-databases/breast-cancer-wisconsin/wdbc.data"
brca_nuclei <- read.csv(url, header = FALSE)

# add variable names - info from wdbc.names file at UCI ML repository
x <- c("radius", "texture", "perimeter", "area", "smoothness", "compactness", "concavity", "concave_pts", "symmetry", "fractal_dim")
y <- c("mean", "se", "worst")
temp <- expand.grid(x = x, y = y)    # build variable names as all combos of x,y
names(brca_nuclei) <- c("id", "type", paste(temp$x, temp$y, sep = "_"))

# arrange by type to group like samples together
brca_nuclei <- brca_nuclei %>% arrange(type)

# extract numeric features
features <- brca_nuclei %>% select(-id, -type) %>% as.matrix()

# extract tumor type
type <- brca_nuclei$type

# combine features and type into brca object
brca <- list(x = features, y = type)

save(brca, file = "data/brca.rda", compress = "xz")

# import greenhouse gas data for Law Dome Ice Core from NOAA
# citation: Etheridge, D.M., et al. 2010

library(tidyverse)

url <- "ftp://ftp.ncdc.noaa.gov/pub/data/paleo/icecore/antarctica/law/law2006.txt"
greenhouse_gases <- read_table(url, skip = 181)[1:2004,] %>%    # keep only the desired table
  select(Year = YearAD, CO2 = CO2spl, CH4 = CH4spl, N2O = N2Ospl) %>%    # rename gases
  gather(key = "Gas", value = "Concentration", -Year)  %>%    # change to tidy format
  filter(Year %in% seq(0, 2000, 20))  %>%  # keep every 20th year to reduce dataset size
  setNames(tolower(names(.)))
  
# save to rda
greenhouse_gases <- as.data.frame(greenhouse_gases) 
save(greenhouse_gases, file = "data/greenhouse_gases.rda", compress = "xz")

#' Italian olive
#' 
#' Composition in percentage of eight fatty acids found in the lipid fraction of 572 Italian olive oils
#' 
#' \itemize{
#' \item region. General region of Italy.
#' \item area. Area of Italy. 
#' \item palmitic. Percent palmitic acid of sample. 
#' \item palmitoleic. Percent palmitoleic of sample. 
#' \item stearic. Percent stearic acid of sample.  
#' \item oleic. Percent oleic acid of sample.  
#' \item linoleic. Percent linoleic acid of sample.  
#' \item linolenic. Percent linolenic acid of sample.  
#' \item arachidic. Percent arachidic acid of sample.  
#' \item eicosenoic. Percent eicosenoic acid of sample.  
#' }
#'
#' @docType data
#'
#' @usage data(olive)
#'
#' @format An object of class \code{"data.frame"}. 
#'
#' @keywords datasets
#'
#'
#' @source J. Zupan, and J. Gasteiger.  Neural Networks in Chemistry and Drug Design.
#' 
#' @examples
#' data(olive)
#' head(olive)
"olive"

source("R/read_mnist.R")
library(caret)

## read-in data
if (!exists("mnist")) mnist <- read_mnist()
## keep only 2s and 7s
index_27 <- which(mnist$train$labels %in% c(2,7))
y <- mnist$train$labels[index_27] 
x <- mnist$train$images[index_27,]

## we start by creating the "true" conditional expectation
row_column <- expand.grid(row=1:28, col=1:28) #temporary object to help figure out the quandrants
upper_left_ind <- which(row_column$col <= 14 & row_column$row <= 14)
lower_right_ind <- which(row_column$col > 14 & row_column$row > 14)
x <- x > 200 #binarize the values. Above 200 is ink, below is no ink
x <- cbind(rowSums(x[ ,upper_left_ind])/rowSums(x), #proportion of pixels in upper right quandrant
           rowSums(x[ ,lower_right_ind])/rowSums(x)) #proportion of pixes in lower rigth quandrant

## To estimate the "true" p(x_1, x_2) we will smooth the result of knn 
fit <-knn3(x, factor(y), 51)
GS <- 125
true_p <- expand.grid(x_1 = seq(min(x[,1]), max(x[,1]), len=GS),
                      x_2 = seq(min(x[,2]), max(x[,2]), len=GS))
                        
y_hat <- predict(fit, newdata = true_p, type="prob")
true_p$p <- pmax(loess(y_hat[,2]~ x_1 * x_2, data= true_p, degree=1, span=1/5)$fitted, 0)

## now we create the example dataset
## it includes the labels, the two predictors, and an index so we can recuperate the image from original mnist data
set.seed(1971)
ind <- sample.int(length(index_27), 1000)
labels <- y[ind]
x <- x[ind, ]
index_train <- createDataPartition(labels, p=0.8, list = FALSE)

mnist_27 <- list(
  train = data.frame(y = factor(labels[index_train]),
                     x_1 = x[index_train,1],
                     x_2 = x[index_train,2]),
  test = data.frame(y = factor(labels[-index_train]),
                     x_1 = x[-index_train,1],
                     x_2 = x[-index_train,2]),
  index_train = index_27[ind][index_train],
  index_test = index_27[ind][-index_train],
  true_p = true_p)

save(mnist_27, file = "data/mnist_27.rda", compress="xz")

# check
#ggplot2::ggplot(mnist_27$true_p, aes(x_1, x_2, fill = p)) + geom_raster()+ scale_fill_gradientn(colors=c("#00BFC4","white","#F8766D"))





       

#' US gun murders by state for 2010
#' 
#' Gun murder data from FBI reports. Also contains the population of each state.
#'
#' \itemize{
#'  \item state. US state
#'  \item abb. Abbreviation of US state
#'  \item region. Geographical US region
#'  \item population. State population (2010)
#'  \item total. Number of gun murders in state (2010)
#' }
#'
#' @docType data
#'
#' @usage data(murders)
#'
#' @format An object of class \code{"data.frame"}. 
#'
#' @keywords datasets
#'
#'
#' @source \href{https://en.wikipedia.org/wiki/Murder_in_the_United_States_by_state}{Wikipedia}
#'
#' @examples
#' data(murders)
#' print(murders)
"murders"

## Extracted from http://www.tylervigen.com/spurious-correlations
divorce_margarine<- data.frame(divorce_rate_maine = c(5, 4.7, 4.6, 4.4, 4.3, 4.1, 4.2, 4.2, 4.2, 4.1),
                   margarine_consumption_per_capita = c(8.2, 7, 6.5, 5.3, 5.2, 4, 4.6, 4.5, 4.2, 3.7),
                   year = 2000:2009) 

save(divorce_margarine, file="data/divorce_margarine.rda", compress="xz", version = 2)

#' Download and read the mnist dataset
#'
#' This function downloads the mnist training and test data available here http://yann.lecun.com/exdb/mnist/
#'
#'
#' 
#' @param path A character giving the full path of the directory to look for files. It assumes the filenames are the same as the originals. If path is \code{NULL} a download or direct read of the files is attempted.
#' @param  download If \code{TRUE} the files will be downloaded and saved in detsdir.
#' @param destdir A character giving the full path of the directory in which to save the downloaded files. The default is to use a temporary directory.
#' @param url A character giving the URL from which to download files. Currently a copy of the data is available at https://www2.harvardx.harvard.edu/courses/IDS_08_v2_03/, the current default URL.
#' @param keep.files A logical. If \code{TRUE} the downloaded files will be saved in \code{destdir}. If \code{FALSE} the entire directory is erased. This argument is ignored if \code{download} is \code{FALSE}.){

#' }
#'
#' @return A list with two components: train and test. Each of these is a list with two components: images and labels. The images component is a
#' matrix with each column representing one of the 28*28 = 784 pixels. The values are integers between 0 and 255 representing grey scale. 
#' The labels components is a vector representing the digit shown in the image.
#'
#' Note that the data is over 10MB, so the download may take several seconds depending on internet speed. If you plan to load the data more than once
#' we recommend you download the data once and read it from disk in the future. See examples.
#' 
#' 
#' @references Y. LeCun, L. Bottou, Y. Bengio, and P. Haffner. "Gradient-based learning applied to document recognition." Proceedings of the IEEE, 86(11):2278-2324, November 1998.  
#' 
#' @author Samuela Pollack
#' @author Rafael A. Irizarry, \email{rafael_irizarry@dfci.harvard.edu}
#' @examples
#' # this can take several seconds, depending on internet speed.
#' 
#' \dontrun{
#' mnist <- read_mnist()
#' i <- 5
#' image(1:28, 1:28, matrix(mnist$test$images[i,], nrow=28)[ , 28:1], 
#'     col = gray(seq(0, 1, 0.05)), xlab = "", ylab="")
#' ## the labels for this image is: 
#' mnist$test$labels[i]
#' }
#'
#' # You can download and save the data to a directory like this:
#' \dontrun{
#' mnist <- read_mnist(download = TRUE, destdir = "~/Downloads")
#' 
#' # and then, going forward, read from disk 
#' mnist <- read_mnist("~/Downloads")
#' }
#' @export
#' 
#' @importFrom utils download.file
#' 
read_mnist <- function(path = NULL, download = FALSE, destdir = tempdir(), 
                       url = "https://www2.harvardx.harvard.edu/courses/IDS_08_v2_03/", keep.files = TRUE){

  mnist <- list(train=list(images=c(), labels=c()), test=list(images=c(), labels=c()))
  for (ttt in c("train", "t10k"))  {
    
    # Open connection to read image file
    fn <- paste0(ttt,"-images-idx3-ubyte.gz")
    
    if(!is.null(path)){
      conn <- gzcon(file(file.path(path, fn), "rb"))
    } else{
      if(download){
        destfile <- file.path(destdir, fn)
        download.file(paste0(url, fn), destfile)
        conn <- gzcon(file(destfile, "rb"))
      } else{
        conn <- gzcon(url(paste0(url, fn), "rb"))
      }
    }
    
    magic <- readBin(conn, 'integer', n = 1, size = 4, endian = "big")  # magic bytes
    typ <- bitwAnd(bitwShiftR(magic,8), 0x000000ff)   # should be 8 => unsigned byte    
    ndm <- bitwAnd(magic, 0x000000ff)                 # dimensionality (should be 3)
    
    # Read dimensions
    dim <- readBin(conn, 'integer', n = ndm, size = 4, endian = "big")
    
    # Read unsigned bytes
    data <- readBin(conn, 'integer', n = prod(dim), size = 1, signed = FALSE)
    
    # store "t10k" data to "test" list
    tt <- ttt
    if(tt == "t10k")
      tt <- "test"
    
    # Save as matrix, row is image number, col is the image as 
    # single-dimensional 28 x 28 array of real.
    mmm <- matrix(data, nrow=dim[1], byrow = TRUE) 
    mnist[[tt]][["images"]] <- mmm
    close(conn)
    
    # Open connection to read label file  
    fn <- paste0(ttt,"-labels-idx1-ubyte.gz")
    
    if(!is.null(path)){
      conn <- gzcon(file(file.path(path, fn), "rb"))
    } else{
      if(download){
        destfile <- file.path(destdir, fn)
        download.file(paste0(url, fn), destfile)
        conn <- gzcon(file(destfile, "rb"))
      } else{
        conn <- gzcon(url(paste0(url, fn), "rb"))
      }
    }
    
    magic <- readBin(conn, 'integer', n = 1, size = 4, endian = "big")  # magic bytes
    nlb <- readBin(conn, 'integer', n = 1, size = 4, endian = "big")    # number of labels
    
    # Read unsigned bytes
    data <- readBin(conn, 'integer', n = nlb, size = 1, signed = FALSE)
    
    # Store as integer
    mnist[[tt]][["labels"]] <- data
  }
  
  if(download){
    if(keep.files){
      message("Keeping downloaded files in the following diretory: ", destdir)
    } else{
      file.remove(file.path(destdir, 
                            c("train-images-idx3-ubyte.gz", "train-labels-idx1-ubyte.gz", 
                              "t10k-images-idx3-ubyte.gz","t10k-labels-idx1-ubyte.gz")))
    }
  }
  
  close(conn)
  return(mnist)
}

## Download data
## remove '^"`  because these are retweets
## change the date a Data object
library(tidyverse)
library(lubridate)
library(tidytext)
library(textdata)

## The data was originally downloaded using the script below. 
## Last time we checked, on 2023/10/30, the website was no longer available.
# url <- 'http://www.trumptwitterarchive.com/data/realdonaldtrump/%s.json'
# trump_tweets <- map(2009:2017, ~sprintf(url, .x)) %>%
#   map_df(jsonlite::fromJSON, simplifyDataFrame = TRUE) %>%
#   filter(!is_retweet & !str_detect(text, '^"')) %>%
#   mutate(created_at = parse_date_time(created_at, orders = "a b! d! H!:M!:S! z!* Y!", tz="EST")) 
#save(trump_tweets, file="data/trump_tweets.rda", compress="xz")

load("data/trump_tweets.rda")

links <- "https://t.co/[A-Za-z\\d]+|&amp;"

nrc <- get_sentiments("nrc") |>
  select(word, sentiment)

sentiment_counts <- trump_tweets |> 
  extract(source, "source", "Twitter for (.*)") |>
  filter(source %in% c("Android", "iPhone") &
           created_at >= ymd("2015-06-17") & 
           created_at < ymd("2016-11-08")) |>
  filter(!is_retweet) |>
  arrange(created_at) |> 
  as_tibble() |>
  mutate(text = str_replace_all(text, links, ""))  |>
  unnest_tokens(word, text) |>
  filter(!word %in% stop_words$word &
           !str_detect(word, "^\\d+$")) |>
  mutate(word = str_replace(word, "^'", "")) |>
  left_join(nrc, by = "word", relationship = "many-to-many") |>
  count(source, sentiment) |>
  pivot_wider(names_from = "source", values_from = "n") |>
  mutate(sentiment = replace_na(sentiment, replace = "none"))


save(trump_tweets, sentiment_counts, file = "data/trump_tweets.rda",
     compress = "xz", version = 2)

#' Gender bias among graduate school admissions to UC Berkeley.
#' 
#' The admission data for six majors for the fall of 1973; often used as an example of Simpson's paradox
#'
#'
#' \itemize{
#'   \item major. The major or university department.
#'   \item gender. Men or women.
#'   \item admitted. Percent of students admitted.
#'   \item applicants. Total number of applicants.
#' }
#' 
#' @docType data
#'
#' @usage data(admissions)
#'
#' @format An object of class \code{"data.frame"}. 
#'
#' @keywords datasets
#'
#' @source  PJ Bickel, EA Hammel, and JW O'Connell. Science (1975)
#' @examples
#' data(admissions)
#' admissions
"admissions"

#' Gapminder Data
#' 
#' Health and income outcomes for 184 countries from 1960 to 2016. Also includes two character vectors, \code{oecd} 
#' and \code{opec}, with the names of OECD and OPEC countries from 2016.
#'
#' \itemize{
#'  \item country.  
#'  \item year.
#'  \item infant_mortality. Infant deaths per 1000.
#'  \item life_expectancy. Life expectancy in years.
#'  \item fertility. Average number of children per woman. 
#'  \item population. Country population.
#'  \item gpd. GDP according to World Bankdev.
#'  \item continent. 
#'  \item region. Geographical region.
#' }
#'
#'
#' @docType data
#'
#' @usage data(gapminder)
#'
#' @format An object of class \code{"data.frame"}. 
#'
#' @keywords datasets
#' 
#' @aliases oecd opec
#'
#' @examples
#' data(gapminder)
#' head(gapminder)
#' print(oecd)
#' print(opec)
"gapminder"

#' Models results from taking a poll
#'
#' The function shows a plot of a random sample drawn from an urn with blue and red beads. The sample is taken with replacement. The proportion of blue beads is not shown so that students can try to estimate it.
#'
#' @param n Sample size
#' @param ... additional arguments to be used by the function \code{sample}.
#'
#' @return None
#'
#' @examples
#' take_poll(25)
#'
#' @export
#'

take_poll <- function(n, ...){
  .take_poll(n, ...)
}

#' Trump Tweets from2009 to 2017
#' 
#' All tweets from Donald Trump's twitter account from 2009 to 2017 
#'
#' \itemize{
#' \item source. Device or service used to compose tweet.
#' \item id_str. Tweet ID.
#' \item text. Tweet.
#' \item created_at. Data and time tweet was tweeted.
#' \item retweet_count. How many times tweet had been retweeted at time dataset was created.
#' \item in_reply_to_user_id_str. If a reply, the user id of person being replied to.
#' \item favorite_count. Number of times tweet had been favored at time dataset was created.
#' \item is_retweet. A logical telling us if it is a retweet or not.
#' }
#' 
#' @docType data
#'
#' @usage data(trump_tweets)
#'
#' @format An object of class \code{"data.frame"}. 
#'
#' @keywords datasets
#'
#' @source  The Trump Twitter Archive: \url{https://www.thetrumparchive.com/}
#' 
#' @examples
#' data(trump_tweets)
#' head(trump_tweets)
#' 
"trump_tweets"

library(readr)
library(tidyverse)
library(stringr)

##polls
url <- "http://projects.fivethirtyeight.com/general-model/president_general_polls_2016.csv"
polls_us_election_2016 <- read_csv(url)
polls_us_election_2016 <- filter(polls_us_election_2016, type=="polls-plus")
polls_us_election_2016$startdate <- as.Date(polls_us_election_2016$startdate,"%m/%d/%Y") ##turn enddate into date
polls_us_election_2016$enddate <- as.Date(polls_us_election_2016$enddate,"%m/%d/%Y") 
polls_us_election_2016 <- select(polls_us_election_2016, state, startdate, enddate, pollster, grade, samplesize, population, rawpoll_clinton, rawpoll_trump, rawpoll_johnson, rawpoll_mcmullin, adjpoll_clinton, adjpoll_trump, adjpoll_johnson, adjpoll_mcmullin)
polls_us_election_2016 <- as.data.frame(polls_us_election_2016)
polls_us_election_2016 <- mutate(polls_us_election_2016, 
                                 state = factor(state), 
                                 pollster = factor(pollster), 
                                 grade=factor(grade, levels = c("D","C-","C","C+","B-","B","B+","A-","A","A+")))

##2016 results
library(googlesheets)
url <- "https://docs.google.com/spreadsheets/d/1zxyOQDjNOJS_UkzerorUCf2OAdcMcIQEwRciKuYBIZ4/pubhtml?widget=true&headers=false#gid=658726802"
key <- extract_key_from_url(url)
gs <- gs_key(key)
tmp <- gs_read(gs, ws = "President", range = cell_cols(c(1:10)))
tmp <- tmp[-c(1:2),]
results_us_election_2016 <- 
  data.frame(state=tmp[[1]], 
             electoral_votes=tmp[[2]],
             clinton=as.numeric(gsub("%","",tmp[[7]])),
             trump=as.numeric(gsub("%","",tmp[[8]])),
             others=as.numeric(gsub("%","",tmp[[9]])),
             stringsAsFactors = FALSE) %>%
  mutate(state = ifelse(state == "Washington, D.C.", "District of Columbia", state))

## we reorder for illustrate how join works
results_us_election_2016 <- results_us_election_2016 %>% arrange(desc(electoral_votes))  
save(polls_us_election_2016, results_us_election_2016, file = "data/polls_us_election_2016.rda", compress="xz")

#' Mice weights 
#' 
#' Body weights, bone density, and percent fat for mice under two diets: chow and high fat. Data provided by Karen Svenson from Jackson Laboratories. Funding to generate these data came from NIH grant P50 GM070683 awarded to Gary Churchill.
#'
#' \itemize{
#'   \item body_weight. Body weight in grams at 19 weeks.
#'   \item bone_density. Body density.
#'   \item percent_fat. Percent fat.
#'   \item sex. The sex of the mice.
#'   \item diet. The diet of the mice: chow or high fat.
#'   \item gen. These are outbread mice. This variable denotes the generation.
#'   \item litter. Which of two litters mice belong to.
#' }
#' 
#' @docType data
#'
#' @usage mice_weights
#'
#' @format An object of class \code{"data.frame"}. 
#'
#' @keywords datasets
#'
#' @source Karen Svenson, Daniel M. Gatti, and Gary Churchill from Jackson Laboratories. 
#' 
#' @references Daniel M. Gatti, Petr Simecek, Lisa Somes, Clifton T. Jeffrey, Matthew J. Vincent, Kwangbom Choi, Xingyao Chen, Gary A. Churchill, and Karen L. Svenson. "The Effects of Sex and Diet on Physiology and Liver Gene Expression in Diversity Outbred Mice". bioRxiv 098657; \doi{10.1101/098657}
#'
#' @examples
#' mice_weights |> head()
#' with(mice_weights, table(sex, diet))
"mice_weights"

#' Poll data for popular vote in 2008 presidential election
#' 
#' Data from different pollsters for the popular vote between Obama and McCain 
#' in the 2008 presidential election.  
#'  
#'
#' \itemize{
#' \item day. Days until election day. Negative numbers are reported so that days can increase up to 0, which is election day.
#' \item margin. Average difference between Obama and McCain for that day.
#' }
#' 
#' @docType data
#'
#' @usage data(polls_2008)
#'
#' @format An object of class \code{data.frame}.
#'  
#' @keywords datasets
#'
#'
#' @source \href{https://web.archive.org/web/20161108190914/http://www.pollster.com/08USPresGEMvO-2.html}{https://web.archive.org/web/20161108190914/http://www.pollster.com/08USPresGEMvO-2.html}
#'
#' @examples
#' data(polls_2008)
#' with(polls_2008, plot(day, margin))
#' 
"polls_2008"

#' 2012 US presidential election results 
#' 
#' Percentages for the four major candidates by state in the US 2012 presidential elections. It includes congressional districts
#' for Maine and Nebraska.
#'
#'
#' \itemize{
#' \item state. State in which poll was taken. `U.S` is for national polls.
#' \item electoral_votes. Electoral votes for that state.
#' \item obama. Percent obtained by Barack Obama.
#' \item romney. Percent obtained by Mitt Romney.
#' \item johnson. Percent obtained by Gary Johnson.
#' \item stein. Percent obtained by Jill Stein.
#' }
#
#' @docType data
#'
#' @usage results_us_election_2012
#'
#' @format An object of class \code{"data.frame"}. 
#'
#' @keywords datasets
#'
#'
#' @source 
#' 
#' Wikepedia: \url{https://en.wikipedia.org/w/index.php?title=2012_United_States_presidential_election&oldid=1264588444}
#'
#'
#' 
#'
#' @examples
#' head(results_us_election_2012)
"results_us_election_2012"

#' Adult male heights in feet with outliers
#' 
#' This dataset was randomly generated with a normal distribution (average: 5 feet 9 inches, standard deviation: 3 inches). One value was changed to be mistakenly reported in centimeters rather than feet.
#'
#'
#' @docType data
#'
#' @usage data(outlier_example)
#'
#' @format An object of class \code{"numeric"}. 
#'
#' @keywords datasets
#'
#'
#'
#' @examples
#' data(outlier_example)
#' mean(outlier_example)
#' median(outlier_example)
"outlier_example"

#' Simulate falling object data
#'
#' The function simulates a falling object's position. Default parameters are for dropping a 
#' weight from the tower of Pisa.
#'
#' @param n Sample size
#' @param d_0 Height from which object will fall in meters.
#' @param v_0 Initial velocity with which object will fall in meters per second.
#' @param g Gravitational constant, 9.8 meters per second per seonnd
#' @param scale The measurement errors will be multiplied by this constant.
#' @param time Numeric vector of times, in seconds, at which measurements were taken.
#' @param error_distribution Character. Either \code{rnorm} for normal or \code{rt} for t-distribution.
#' @param df If using t-distribution, the degrees of freedom.
#' 
#' @return A \code{data.frame} with the time, the distance travelled, and the observed distance.
#'
#' @examples
#' 
#' dat <- rfalling_object()
#' with(dat, plot(time, observed_distance))
#' with(dat, lines(time, distance, col = "blue"))
#'
#' @importFrom stats rnorm rt
#' 
#' @export
#'

rfalling_object <- function(n = 14, d_0 = 55.86, v_0 = 0, g = -9.8,  
                           scale = 1,
                           time = seq(0, 3.25, length.out = n),
                           error_distribution = c("rnorm", "rt"),
                           df = 3){
  error_distribution <- match.arg(error_distribution)
  error_func = get(error_distribution)
  
  if(length(time)!=n) stop("length(time) must be equal to n")
  d <- d_0 + v_0 * time + 0.5*g*time^2
  
  if(error_distribution == "rnorm"){
    y <- d  + rnorm(n)*scale
  } else{
    y <- d + rt(n, df = df) * scale
  }
  
  dat <- data.frame(time = time, distance = pmax(d,0), observed_distance = pmax(y,0))
  
  attr(dat, "params")  <- c(d_0 = d_0, v_0 = v_0, g = g, scale = scale)
  attr(dat, "error_distribution") <- error_distribution
  
  dat 
}

#' Atmospheric carbon dioxide concentration over 800,000 years
#' 
#' Concentration of carbon dioxide in ppm by volume from direct measurements at Mauna Loa (1959-2018 CE) and indirect measurements from a series of Antarctic ice cores (approx. -800,000-2001 CE).
#' 
#'
#' \itemize{
#'  \item year. Year (CE).
#'  \item co2. Carbon dioxide concentration in ppm by volume.
#'  \item source. Source of carbon dioxide measurement: direct CO2 annual mean concentrations from Mauna Loa (`Mauna Loa`) or indirect CO2 concentrations from air trapped in ice cores (`Ice Cores`).
#' }
#'
#' @docType data
#'
#' @usage data(historic_co2)
#'
#' @format An object of class \code{"data.frame"}. 
#'
#' @keywords datasets
#'
#'
#' @source Mauna Loa data from \href{https://www.esrl.noaa.gov/gmd/ccgg/trends/data.html}{NOAA}. Ice core data from Bereiter et al. 2015 via \href{https://www.ncdc.noaa.gov/paleo-search/study/17975}{NOAA}.
#' 
#'
#' @examples
#' data(historic_co2)
#' head(historic_co2)
"historic_co2"

#' Physical Properties of Stars
#' 
#' Physical properties of selected stars, including luminosity, temperature, and spectral class.
#'
#'
#' \itemize{
#'  \item star. Name of star.
#'  \item magnitude. Absolute magnitude of the star, which is a function of the star's luminosity and distance to the star.
#'  \item temp. Surface temperature in degrees Kelvin (K).
#'  \item type. Spectral class of star in the OBAFGKM system.
#' }
#'
#' @docType data
#'
#' @usage data(stars)
#'
#' @format An object of class \code{"data.frame"}. 
#'
#' @keywords datasets
#'
#'
#' @source Compiled from multiple open-access references on \href{http://vizier.u-strasbg.fr/viz-bin/VizieR}{VizieR}.
#'
#' @examples
#' data(stars)
#' head(stars)
"stars"

.take_poll <- function(n, plot = TRUE, return_values = FALSE, blue_prob = 0.529, jitter_factor = 1, cex=1.2, ...){
  bead_color <- sample(c("Blue","Red"), n, prob=c(blue_prob, 1-blue_prob), replace = TRUE)
  if(plot){
    cols <- ifelse(bead_color=="Blue", "#0072B2", "#D55E00")
    x <- jitter(ifelse(bead_color=="Blue", 1, 2), factor = jitter_factor)
    y <- stats::rnorm(n)
    plot(x,y, pch=21, bg=cols, xlim=c(0.5,2.5), xlab="", ylab="", xaxt="n", yaxt="n", cex = cex, bty = "n", ...)
    graphics::axis(side=3, at=c(1,2), labels = c("Blue", "Red"), tick=FALSE)
    }
  if(return_values) return(bead_color) 
}

#' Fivethirtyeight 2016 Poll Data 
#' 
#' Poll results from US 2016 presidential elections aggregated from HuffPost Pollster, RealClearPolitics, polling firms, and news reports. 
#' The dataset also includes election results (popular vote) and electoral college votes in \code{results_us_election_2016}.
#'
#'
#' \itemize{
#' \item state. State in which poll was taken. `U.S` is for national polls.
#' \item startdate. Poll's start date.
#' \item enddate. Poll's end date.
#' \item pollster. Pollster conducting the poll.
#' \item grade. Grade assigned by fivethirtyeight to pollster.
#' \item samplesize. Sample size. 
#' \item population. Type of population being polled.
#' \item rawpoll_clinton. Percentage for Hillary Clinton.
#' \item rawpoll_trump. Percentage for Donald Trump
#' \item rawpoll_johnson. Percentage for Gary Johnson
#' \item rawpoll_mcmullin. Percentage for Evan McMullin.
#' \item adjpoll_clinton. Fivethirtyeight adjusted percentage for Hillary Clinton.
#' \item ajdpoll_trump. Fivethirtyeight adjusted percentage for Donald Trump
#' \item adjpoll_johnson. Fivethirtyeight adjusted percentage for Gary Johnson
#' \item adjpoll_mcmullin. Fivethirtyeight adjusted percentage for Evan McMullin.
#' }
#
#' @docType data
#'
#' @usage data(polls_us_election_2016)
#'
#' @format An object of class \code{"data.frame"}. 
#'
#' @keywords datasets
#'
#'
#' @source 
#' 
#' The original csv file used to create \code{polls_us_election_2016} is here: \url{https://projects.fivethirtyeight.com/general-model/president_general_polls_2016.csv} 
#' 
#' The data for \code{results_us_election_2016} is from Ballotpedia and can be found here:
#' \url{https://docs.google.com/spreadsheets/d/1zxyOQDjNOJS_UkzerorUCf2OAdcMcIQEwRciKuYBIZ4/pubhtml?widget=true&headers=false#gid=658726802/}
#'
#'
#' @aliases results_us_election_2016
#' 
#'
#' @examples
#' data(polls_us_election_2016)
#' head(polls_us_election_2016)
"polls_us_election_2016"

#' Self-reported Heights in Inches
#' 
#' Self-reported heights and sex. The heights were converted to inches from the original data included in \code{\link{reported_heights}}.
#' 
#'\itemize{
#'  \item sex. A factor with the self-reported sex. 
#'  \item height. A numeric vector with self-reported heights in inches. 
#' }
#'
#'
#' @seealso
#' \code{\link{reported_heights}} for the original data source.
#' 
#' @docType data
#'
#' @usage heights
#'
#' @format An object of class \code{"data.frame"}. 
#'
#' @keywords datasets
#'
#'
#'
#' @examples
#' head(heights)
"heights"

#' dslabs theme set 
#'
#' This function sets a ggplot2 theme used throughout the data science labs. It can be called without arguments.
#'
#' @param new a prebuilt ggplot2 theme. Defaults to \code{"theme_minimal"}
#' @param args the arguments to be passed along to the ggplot2 theme function. Defaults to \code{"NULL"}.
#' @param base_size if \code{"args"} is \code{"NULL"}, base_size is one of the arguments passed to the theme function. It defaults to 11.
#' @param bold_title if TRUE, sets titles to be bold
#' @param ... additional arguments to be used by theme
#'  
#' @return None
#'
#' @examples
#' library(ggplot2)
#' ds_theme_set()
#' qplot(hp, mpg, data=mtcars, color=am, facets=gear~cyl, 
#' main="Scatterplots of MPG vs. Horsepower", 
#' xlab="Horsepower", ylab="Miles per Gallon")
#'
#' @export
#' 
ds_theme_set <- function(new = "theme_bw", args = NULL,
                         base_size = 11, 
                         bold_title = TRUE, 
                         ...){
  requireNamespace("ggplot2")
  
  if(! "package:ggplot2" %in% search() ) stop("ggplot2 must be loaded.\nLoad with library(ggplot2) or library(tidyverse).")
  
  if(is.null(args)) args <- list(base_size=base_size)
  
  th <- do.call(new, args) + ggplot2::theme(...)
  ggplot2::theme_set(th)
  
  if(bold_title) ggplot2::theme_update(plot.title=ggplot2::element_text(face="bold"))
}

#' Useful example for illustrating machine learning algorithms based on MNIST data
#' 
#' We only include a randomly selected set of 2s and 7s along with the two predictors based on the 
#' proportion of dark pixels in the upper left and lower right quadrants respectively. The dataset is divided into 
#' training and test sets.
#'  
#'
#' \itemize{
#' \item train. A data frame containing  training data: labels and predictors.
#' \item test. A data frame containing  test data: labels and predictors.
#' \item index_train. The index of the original mnist training data used for the training set.
#' \item index_test. The index of the original mnist test data used for the test set.
#' \item true_p. A \code{data.frame} containing the 
#' two predictors \code{x_1} and \code{x_2} and the conditional probability of being a 7 
#' for \code{x_1}, \code{x_2}.
#' }
#' 
#' @seealso [read_mnist()]
#' 
#' @docType data
#'
#' @usage data(mnist_27)
#'
#' @format An object of class \code{list}.
#'  
#' @keywords datasets
#'
#' @references Y. LeCun, L. Bottou, Y. Bengio, and P. Haffner. "Gradient-based learning applied to document recognition." Proceedings of the IEEE, 86(11):2278-2324, November 1998.  
#'
#'
#' @examples
#' data(mnist_27)
#' with(mnist_27$train, plot(x_1, x_2, col = as.numeric(y)))
#' 
"mnist_27"

#' Gender bias in research funding in the Netherlands
#' 
#' Table S1 from paper title "Gender contributes to personal research funding success in The Netherlands"
#'
#'
#' \itemize{
#'   \item discipline. Research area discipline.
#'   \item applications_total. Total applications.
#'   \item applications_men. Total applications by men.
#'   \item applications_women. Total applications by women.
#'   \item awards_total. Total awards.
#'   \item awards_men. Total awards received by men.
#'   \item awards_women. Total awards received by women.
#'   \item success_rates_total. Overall success rate.
#'   \item success_rates_men. Success rate for men.
#'   \item success_rates_women. Success rate for women.
#' }
#' 
#' @docType data
#'
#' @usage research_funding_rates
#'
#' @format An object of class \code{"data.frame"}. 
#'
#' @keywords datasets
#'
#' @references van der Lee R, Ellemers N. Gender contributes to personal research funding success in The Netherlands. Proc Natl Acad Sci U S A. 2015 Oct 6;112(40):12349-53. doi: 10.1073/pnas.1510159112. Epub 2015 Sep 21. PMID: 26392544; PMCID: PMC4603485.
#' 
#' @aliases raw_data_research_funding_rates
#' 
#' @examples

#' research_funding_rates
#' # The raw data for this table is available from
#' invisible(raw_data_research_funding_rates)
#' 
"research_funding_rates"

#' Useful example for illustrating generative models based on MNIST data
#' 
#' A randomly selected set of 1s, 2s and 7s along with the two predictors based on the 
#' proportion of dark pixels in the upper left and lower right quadrants respectively. The dataset is divided into 
#' training and test sets.
#'  
#'
#' \itemize{
#' \item train. A data frame containing  training data: labels and predictors.
#' \item test. A data frame containing  test data: labels and predictors.
#' }
#' 
#' @seealso [read_mnist(), mnist_27]
#' 
#' @docType data
#'
#' @usage mnist_127
#'
#' @format An object of class \code{list}.
#'  
#' @keywords datasets
#'
#' @references Y. LeCun, L. Bottou, Y. Bengio, and P. Haffner. "Gradient-based learning applied to document recognition." Proceedings of the IEEE, 86(11):2278-2324, November 1998.  
#'
#'
#' @examples
#' with(mnist_127$train, plot(x_1, x_2, col = as.numeric(y)))
#' 
"mnist_127"

#' Brexit Poll Data
#' 
#' Brexit (EU referendum) poll outcomes for 127 polls from January 2016 to the referendum date on June 23, 2016.
#'
#'
#' \itemize{
#'  \item startdate. Start date of poll.
#'  \item enddate. End date of poll.
#'  \item pollster. Pollster conducting the poll.
#'  \item poll_type. Online or telephone poll.
#'  \item samplesize. Sample size of poll.
#'  \item remain. Proportion voting Remain.
#'  \item leave. Proportion voting Leave.
#'  \item undecided. Proportion of undecided voters.
#'  \item spread. Spread calculated as remain - leave.
#' }
#'
#'
#' @docType data
#'
#' @usage data(brexit_polls)
#'
#' @format An object of class \code{"data.frame"}. 
#'
#' @keywords datasets
#' 
#' @source \href{https://en.wikipedia.org/w/index.php?title=Opinion_polling_for_the_United_Kingdom_European_Union_membership_referendum&oldid=896735054}{Wikipedia}
#'
#' @examples
#' data(brexit_polls)
#' head(brexit_polls)
"brexit_polls"

#' Self-reported Heights 
#' 
#' Students were asked to report their height (in inches) and sex in an online form. 
#' This table includes the results from four courses.
#'
#'\itemize{
#'  \item time_stamp. Time and date of the entry.
#'  \item sex. Sex of the student.
#'  \item height. Height as reported by student by filling in a text free box. 
#' }
#'
#'
#' @docType data
#'
#' @usage data(reported_heights)
#'
#' @format An object of class \code{"data.frame"}. 
#'
#' @keywords datasets
#'
#'
#'
#' @examples
#' data(reported_heights)
#' head(reported_heights)
"reported_heights"

#' Breast Cancer Wisconsin Diagnostic Dataset from UCI Machine Learning Repository
#' 
#' Biopsy features for classification of 569 malignant (cancer) and benign (not cancer) breast masses.
#' 
#' Features were computationally extracted from digital images of fine needle aspirate
#' biopsy slides. Features correspond to properties of cell nuclei, such as size, shape and regularity.
#' The mean, standard error, and worst value of each of 10 nuclear parameters is reported for a total of 30 features.
#' 
#' This is a classic dataset for training and benchmarking machine learning algorithms.
#'
#'
#' \itemize{
#' \item y. The outcomes. A factor with two levels denoting whether a mass is malignant ("M") or benign ("B").
#' \item x. The predictors. A matrix with the mean, standard error and worst value of each of 10 nuclear measurements on the slide, for 30 total features per biopsy:
#'     \itemize{
#'       \item radius. Nucleus radius (mean of distances from center to points on perimeter).
#'       \item texture. Nucleus texture (standard deviation of grayscale values).
#'       \item perimeter. Nucleus perimeter.
#'       \item area. Nucleus area.
#'       \item smoothness. Nucleus smoothness (local variation in radius lengths).
#'       \item compactness. Nucleus compactness (perimeter^2/area - 1).
#'       \item concavity, Nucleus concavity (severity of concave portions of the contour).
#'       \item concave_pts. Number of concave portions of the nucleus contour.
#'       \item symmetry. Nucleus symmetry.
#'       \item fractal_dim. Nucleus fractal dimension ("coastline approximation" -1).
#'     }
#' }
#' 
#
#' @docType data
#'
#' @usage data(brca)
#'
#' @format An object of class \code{list}.
#'  
#' @keywords datasets
#'
#'
#' @source \href{https://archive.ics.uci.edu/ml/datasets/}{UCI Machine Learning Repository}
#' 
#'
#' @examples
#' data(brca)
#' table(brca$y)
#' dim(brca$x)
#' head(brca$x)
"brca"

## Download data
## remove '^"`  because these are retweets
## change the date a Data object
library(tidyverse)
library(lubridate)

url <- 'http://www.trumptwitterarchive.com/data/realdonaldtrump/%s.json'
trump_tweets <- map(2009:2017, ~sprintf(url, .x)) %>%
  map_df(jsonlite::fromJSON, simplifyDataFrame = TRUE) %>%
  filter(!is_retweet & !str_detect(text, '^"')) %>%
  mutate(created_at = parse_date_time(created_at, orders = "a b! d! H!:M!:S! z!* Y!", tz="EST")) 
  
save(trump_tweets, file="data/trump_tweets.rda", compress="xz")

#' Count data with some missing values
#' 
#' This dataset was randomly generated.
#'
#'
#' @docType data
#'
#' @usage data(na_example)
#'
#' @format An object of class \code{"integer"}. 
#'
#' @keywords datasets
#'
#'
#'
#' @examples
#' data(na_example)
#' print(sum(is.na(na_example)))
"na_example"

#' Gene expression profiles for 189 biological samples taken from seven different tissue types.
#' 
#' This is a subset of the data provided by the \code{tissuesGeneExpression}
#' package available from the \code{genomicsclass} GitHub repository. 

#' The predictors are gene expression measurements from 500 genes that
#' are a random subset of the original 22,215. 
#' 
#' The example dataset is recommended for illustrating
#' clustering and machine learning techniques.
#'
#'
#' \itemize{
#' \item x. The predictors composed of 500 genes. Each row is a gene expression profile and each column is different gene. The column names are the gene symbols.
#' \item y. The outcomes. A character vector representing the tissue. One of seven tissue types.
#' }
#' 
#
#' @docType data
#'
#' @usage data(tissue_gene_expression)
#'
#' @format An object of class \code{list}.
#'  
#' @keywords datasets
#'
#'
#' @source \href{https://github.com/genomicsclass/tissuesGeneExpression}{https://github.com/genomicsclass/tissuesGeneExpression}
#' 
#'
#' @examples
#' data(tissue_gene_expression)
#' table(tissue_gene_expression$y)
#' dim(tissue_gene_expression$x)
"tissue_gene_expression"

#' Movie ratings 
#' 
#' MovieLens Latest Dataset (Small)
#'
#' \itemize{
#' \item movieId. Unique ID for the movie.
#' \item title. Movie title (not unique). 
#' \item year. Year the movie was released.
#' \item genres. Genres associated with the movie.
#' \item userId. Unique ID for the user. 
#' \item rating. A rating between 0 and 5 for the movie.
#' \item timestamp. Date and time the rating was given.
#' }
#
#' @docType data
#' 
#' @usage data(movielens)
#'
#' @format Two object of class \code{data.frame}. 
#'
#' @keywords datasets
#' 
#' @references 
#' F. Maxwell Harper and Joseph A. Konstan. 2015. The MovieLens Datasets: History and Context. ACM Transactions on Interactive Intelligent Systems (TiiS) 5, 4, Article 19 (December 2015), 19 pages. DOI=http://dx.doi.org/10.1145/2827872
#' 
#' @source \href{http://files.grouplens.org/datasets/movielens/ml-latest-small.zip}{http://files.grouplens.org/datasets/movielens/ml-latest-small.zip}
#' 
#' @examples
#' data(movielens)
#' head(movielens)
"movielens"

#' 2015 US Period Life Table
#' 
#' Probability of death within 1 year by age and sex in the United States in 2015.
#' 
#'
#' \itemize{
#'  \item age. Age strata, with each year a different stratum.
#'  \item sex. Male or Female.
#'  \item prob. Probability of death within 1 year given exact age and sex.
#' }
#'
#' @docType data
#'
#' @usage data(death_prob)
#'
#' @format An object of class \code{"data.frame"}. 
#'
#' @keywords datasets
#'
#'
#' @source \href{https://www.ssa.gov/oact/STATS/table4c6.html}{Social Security Administraton}
#' @examples
#' data(death_prob)
#' head(death_prob)
"death_prob"

#' Fit a Latent Factor Recommender Model via Alternating Least Squares
#'
#' This function fits a penalized latent factor model for recommendation systems
#' using an alternating least squares (ALS) algorithm. It estimates user effects,
#' item effects, and latent factors simultaneously, with regularization to
#' prevent overfitting. The implementation supports filtering out items with too
#' few ratings.  
#' 
#' @param rating A numeric vector of observed ratings.
#' @param user_id A character vector identifying the user
#'   for each rating. Must be the same length as `rating`.
#' @param item_id A character vector identifying the item
#'   for each rating. Must be the same length as `rating`.
#' @param K Integer. The number of latent factors to estimate.
#' @param lambda_1 Numeric. Regularization parameter for user and item effects.
#' @param lambda_2 Numeric. Regularization parameter for latent factors.
#' @param min_ratings Integer. Minimum number of ratings required for an item to
#'   be included in the estimation of latent factors.
#' @param maxit Integer. Maximum number of iterations.
#' @param reltol Numeric. Relative reltolerance for convergence, based on change in
#'   the objective function.
#' @param damping Numeric between 0 and 1. Damping factor used to blend updates
#'   with the previous iteration for convergence stability.
#' @param verbose Logical. If `TRUE`, prints progress messages during
#'   optimization.
#'   
#' @return A list with the following components:
#'   \item{mu}{Global mean rating.}
#'   \item{a}{Named numeric vector of user-specific effects.}
#'   \item{b}{Named numeric vector of item-specific effects.}
#'   \item{p}{Matrix of user latent factors, with one row per user. The rownames of this matrix match the names of `a`.}
#'   \item{q}{Matrix of item latent factors, with one row per item. The rownames of this matrix match the names of `b`.}
#'   \item{min_ratings}{The threshold value used to filter items: only items with at least this many ratings are included in the estimation of latent factors.}
#'   \item{n_item}{Named integer vector of number of ratings per item.}
#'   \item{n_user}{Named integer vector of number of retained ratings per user.}
#'   \item{fitted}{Fitted values.}
#'
#'
#' @details
#' The model being fit is:
#' \deqn{Y_{ij} = \mu + \alpha_i + \beta_j + \sum_{k=1}^K p_{ik} q_{jk} + \varepsilon_{ij}}
#' where \eqn{\mu} is the global mean, \eqn{\alpha_i} are user effects,
#' \eqn{\beta_j} are item effects, and the \eqn{p_{ik}}, \eqn{q_{jk}} are latent
#' factors for users and items respectively. The estimation minimizes a penalized
#' least squares criterion with separate penalties for user/item effects and
#' latent factors.
#' 
#' Items with less than min_ratings observations are excluded from the estimation 
#' of `p` and `q`.
#'
#' @examples
#' set.seed(2010)
#' ## Simulation settings
#' n_users <- 200       # number of users
#' n_items <- 300       # number of items
#' K_true  <- 4         # true number of latent factors
#' sparsity <- 0.25     # ~5% of user-item pairs are observed
#' 
#' ## True parameters
#' mu <- 3.5
#' a_true <- rnorm(n_users, 0, 0.3)     # user effects
#' b_true <- rnorm(n_items, 0, 0.4)     # item effects
#' p_true <- matrix(rnorm(n_users * K_true, 0, 0.5), n_users, K_true)
#' q_true <- matrix(rnorm(n_items * K_true, 0, 0.5), n_items, K_true)
#'
#' names(a_true) <- 1:n_users
#' names(b_true) <- 1:n_items
#' rownames(p_true) <- 1:n_users
#' rownames(q_true) <- 1:n_items
#' ## Generate observed ratings matrix with sparsity
#' user_id <- rep(as.character(1:n_users), each = n_items)
#' item_id <- rep(as.character(1:n_items), times = n_users)
#' 
#' 
#' ## Which entries are observed?
#' obs <- runif(length(user_id)) < sparsity
#' 
#' ## Ratings with noise
#' rating_full <- mu + a_true[user_id] + b_true[item_id] +
#'   rowSums(p_true[user_id, ] * q_true[item_id, ]) +
#'   rnorm(length(user_id), 0, 0.25)
#' 
#' rating <- rating_full[obs]
#' user_id <- user_id[obs]
#' item_id <- item_id[obs]
#' 
#' ## Call your recommender function
#' fit <- fit_recommender_model(rating, user_id, item_id, K = 4, reltol = 1e-5,
#'                              min_ratings = 5, verbose = TRUE)
#' plot(fit$fitted, rating)
#'
#' @export

fit_recommender_model <- function(rating, user_id, item_id,
                                  K = 8,
                                  lambda_1 = 0.00005,
                                  lambda_2 = 0.0001,
                                  min_ratings = 20,
                                  maxit = 500,
                                  reltol = 1e-8,
                                  damping = 0.75,
                                  verbose = FALSE){
  
  if (!all.equal(length(rating), length(user_id), length(item_id)))
    stop("rating, userId, and item_id must be the same length.")
  
  ## make sure they are character since they are used as names
  user_id <- as.character(user_id) 
  item_id <- as.character(item_id)
  N <- length(rating)
  
  ## index of indexes for each user and item
  indexes_u <- split(1:N, user_id)
  indexes_i <- split(1:N, item_id)
  
  ## Number of ratings per item
  n_item <- sapply(indexes_i, length)
  ## index of items with at least min_ratings
  index_q <- which(n_item >= min_ratings)
  
  ## The n_item associated with each observed rating
  ns <- n_item[item_id]
  
  ## indices of data points whose items have at least min_ratings
  ind <- which(ns >= min_ratings)
  ## If users ends up with less than min_ratings we ask they be excluded
  n_user <- table(as.factor(user_id)[ind])
  if (any(n_user == 0)) {
    stop("After filtering out movies with too few ratings, no ratings remain for the following user(s): ", 
         paste(names(n_user)[n_user == 0], collapse = ", "))
  } 
  if (min(n_user) < min_ratings) {
    warning("After filtering out items with too few ratings, one or more users no longer have more than min_ratings observation. Use the `n_user` component to identify the affected user(s).")
  }
  ## speed-up: index only observations that survive the min_ratings filter (avoid implicit zeros)
  indexes_u_nozeros <- split(ind, user_id[ind])
  indexes_i_q <- indexes_i[index_q]
    
  ## starting values for global mean (mu), user effects (a), movie/item effects (b)
  mu <- mean(rating)
  a <- sapply(indexes_u, function(i) sum(rating[i] - mu)/(length(i) + lambda_1*N))
  resid <- rating - mu - a[user_id]
  b <- sapply(indexes_i, function(i) sum(resid[i])/(length(i) + lambda_1*N))
  
  I <- length(indexes_u)
  J <- length(indexes_i)
  p <- svd(matrix(rnorm(K*I, 0, 0.1), I, K))$u
  rownames(p) <- names(indexes_u)
  q <- matrix(rep(0, K*J), J, K)
  rownames(q) <- names(indexes_i)
  pq <- rep(0, N)
  prev_obj <- 0
  
  for (iter in 1:maxit) {
    ## estimate user and movie effects
    resid <- rating - (mu + b[item_id] + pq)
    a <- sapply(indexes_u, function(i) sum(resid[i])/(length(i) + lambda_1*N))
    resid <- rating - (mu + a[user_id] + pq)
    b <- sapply(indexes_i, function(i) sum(resid[i])/(length(i) + lambda_1*N))
    
    ## Now estimate latent factors p (users) and q (items)
    ## ALS updates one factor (column k) at a time, holding others fixed
    ## treating all the others as constant.
    ## closed-form ridge update using only observed entries (sparse)
    prev_p <- p ## prev_p and prev_q are defined for damping
    prev_q <- q 
    pq <- rowSums(p[user_id, -1, drop = FALSE] * q[item_id, -1, drop = FALSE])
    resid <- rating - (mu + a[user_id] + b[item_id] + pq)
    for (k in 1:K) {
      ## update q 
      q[index_q, k] <- sapply(indexes_i_q, function(i) { 
        x <- p[user_id[i], k]
        sum(x*resid[i])/(sum(x^2) + N*lambda_2)
      })
      q[,k] <- damping*q[,k] + (1 - damping)*prev_q[,k]
      
      ## update p
      p[,k] <- sapply(indexes_u_nozeros, function(i) {
        x <- q[item_id[i], k]
        sum(x*resid[i])/(sum(x^2) + N*lambda_2)
      })
      p[,k] <- damping*p[,k] + (1 - damping)*prev_p[,k]
      
      ## update residual to reflect the newly updated p[,k] and q[,k]
      resid <- resid - p[user_id, k]*q[item_id, k]
    }
    ## Build final latent interaction term pq = row-wise sum of p[user]*q[item]
    pq <- rowSums(p[user_id, ] * q[item_id, ])
    
    resid <- rating - (mu + a[user_id] + b[item_id] + pq)
    
    obj <- mean(resid^2) + 
      lambda_1*(sum(a^2) + sum(b^2)) + 
      lambda_2*(sum(p^2) + sum(q^2))
    
    rel_change <- (prev_obj - obj) / (prev_obj + 1e-8)
    if (verbose) {
      message(sprintf("Iteration %d: Objective = %.6f, Relative = %.6f", 
                      iter, obj, ifelse(iter > 1, rel_change, NA)))
    }
    prev_obj <- obj
    if (abs(rel_change) < reltol) break
  }

  
  ## orthogonalize factors via SVD of p %*% t(q[index_q,]) and rescale by sqrt(s$d)
  QR_p <- qr(p)
  QR_q <- qr(q[index_q,,drop = FALSE])
  s <- svd(qr.R(QR_p) %*% t(qr.R(QR_q)))
  u <- qr.Q(QR_p) %*% s$u
  v <- qr.Q(QR_q) %*% s$v
  
  rownames(u) <- rownames(p)
  rownames(v) <- rownames(q[index_q,,drop = FALSE])
  p <- sweep(u, 2, sqrt(s$d), FUN = "*")
  q[index_q,] <- sweep(v, 2, sqrt(s$d), FUN = "*")
  
  return(list(mu = mu, a = a, b = b, p = p, q = q, 
              min_ratings = min_ratings,
              n_item = sapply(indexes_i, length),
              n_user = n_user,
              fitted = mu + a[user_id] + b[item_id] + pq))
}

#' Divorce rate and margarine consumption data
#' 
#' Divorce rates in Maine and per capita consumption of margarine in US data
#' 
#' 
#'
#' \itemize{
#'   \item divorce_rate_maine. Divorce per 1000 in Maine.
#'   \item margarine_consumption_per_capita. US per capita consumption of margarine in pounds.
#'   \item year. Year.
#' }
#'
#' @docType data
#'
#' @usage data(divorce_margarine)
#'
#' @format An object of class \code{"data.frame"}.
#'
#' @keywords datasets
#'
#' @source \href{http://www.tylervigen.com/spurious-correlations}{Spurious Correlations}
#'
#' @examples
#' data(divorce_margarine)
#' with(divorce_margarine, plot(margarine_consumption_per_capita, divorce_rate_maine))
"divorce_margarine"

##Set working directory to package main directory
library(dplyr)
library(rvest)
library(readr)
library(stringr)

### US murders 2010

url <- "https://en.wikipedia.org/wiki/Murder_in_the_United_States_by_state"
h <- read_html(url)
us_murders <- h %>% html_nodes("table") %>%  
  html_table %>% .[[1]] %>% 
  tbl_df %>% setNames(c("state", "population", "population_density", 
                        "murders", "gun_murders", "gun_ownership", "murder_rate", "gun_murder_rate"))

to_parse_number <- names(us_murders)[-1]
us_murders %>% mutate_at(to_parse_number, parse_number)

columns_to_add <- data_frame(state = c(state.name, "District of Columbia"),
                             abb = c(state.abb,"DC"),
                             region = factor( c(as.character(state.region), "South"), 
                                              levels = levels(state.region)))

us_murders <- full_join(us_murders, columns_to_add, by = "state")

murders <- with(us_murders,
                data.frame(state = state,
                      abb = abb,
                      region = region,
                      population = parse_number(population),
                      total = parse_number(us_murders$gun_murders),
                      stringsAsFactors = FALSE))

save(murders, file = "data/murders.rda")
                

#' Puerto Rico daily mortality
#' 
#' A data frame with Puerto Rico daily mortality counts 2015 to May 2018.
#' This includes the day hurricanes Maria made 2017-09-20. 
#' 
#' \itemize{
#' \item date. Date of the count.
#' \item deaths. Number of deaths reported that day.
#' }
#' 
#' @format An object of class \code{data.frame}.
#'  
#' @source Puerto Rico Demographic Registry. Data was extracted from PDF provided in `system.file("extdata", "RD-Mortality-Report_2015-18-180531.pdf", package = "dslabs")`
#' 
#' @usage pr_death_counts
#' 
#' @docType data
#'
#' @keywords datasets
#'
#' @examples 
#' 
#' with(pr_death_counts, plot(date, deaths))
#' 
"pr_death_counts"

#' Global temperature anomaly and carbon emissions, 1751-2018
#' 
#' Annual mean global temperature anomaly on land, sea and combined, 1880-2018. Annual global carbon emissions, 1751-2014.
#' 
#'
#' \itemize{
#'  \item year. Year (CE).
#'  \item temp_anomaly. Global annual mean temperature anomaly in degrees Celsius relative to the 20th century mean temperature. 1880-2018. 
#'  \item land_anomaly. Annual mean temperature anomaly on land in degrees Celsius relative to the 20th century mean temperature. 1880-2018.
#'  \item ocean_anomaly. Annual mean temperature anomaly over ocean in degrees Celsius relative to the 20th century mean temperature. 1880-2018.
#'  \item carbon_emissions. Annual carbon emissions in millions of metric tons of carbon. 1751-2014.
#' }
#'
#' @docType data
#'
#' @usage data(temp_carbon)
#'
#' @format An object of class \code{"data.frame"}. 
#'
#' @keywords datasets
#'
#'
#' @source \href{https://www.ncdc.noaa.gov/cag/global/time-series}{NOAA} and Boden, T.A., G. Marland, and R.J. Andres (2017) via \href{https://cdiac.ess-dive.lbl.gov/trends/emis/tre_glob_2014.html}{CDIAC}
#'
#' @examples
#' data(temp_carbon)
#' head(temp_carbon)
"temp_carbon"

#' NYC Regents exams scores 2010
#' 
#' Distribution of scores for New York City Regents algebra, global history, biology, English, and U.S. history exams. 
#' These data were used to make \href{http://graphics8.nytimes.com/images/2011/02/19/nyregion/19schoolsch/19schoolsch-popup.gif}{this New York Times plot}.
#' 
#' \itemize{
#' \item score. Test score from 0 to 100.
#'  \item integrated_algebra. Score frequency on Algebra exam.
#'  \item global_history. Score frequency on global history exam.
#'  \item living_environment.  Score frequency on biology exam.
#'  \item english. Score frequency on English exam.
#'  \item us_history. Score frequency on  U.S. history exam.
#' }
#'
#' @docType data
#'
#' @usage data(nyc_regents_scores)
#'
#' @format An object of class \code{"data.frame"}. 
#'
#' @keywords datasets
#'
#'
#' @source New York City Department of Education via Amanda Cox.
#'
#' @examples
#' data(nyc_regents_scores)
#' print(nyc_regents_scores)
"nyc_regents_scores"

source("R/read_mnist.R")
library(caret)

## read-in data
mnist <- read_mnist()
## keep only 2s and 7s
index_27 <- which(mnist$train$labels %in% c(2,7))
y <- mnist$train$labels[index_27] 
x <- mnist$train$images[index_27,]

## we start by creating the "true" conditional expectation
row_column <- expand.grid(row=1:28, col=1:28) #temporary object to help figure out the quandrants
upper_left_ind <- which(row_column$col <= 14 & row_column$row <= 14)
lower_right_ind <- which(row_column$col > 14 & row_column$row > 14)
x <- x > 200 #binarize the values. Above 200 is ink, below is no ink
x <- cbind(rowSums(x[ ,upper_left_ind])/rowSums(x), #proportion of pixels in upper right quandrant
           rowSums(x[ ,lower_right_ind])/rowSums(x)) #proportion of pixes in lower rigth quandrant

## To estimate the "true" p(x_1, x_2) we will smooth the result of knn 
fit <-knn3(x, factor(y), 51)
GS <- 150
true_p <- expand.grid(x_1 = seq(min(x[,1]), max(x[,1]), len=GS),
                      x_2 = seq(min(x[,2]), max(x[,2]), len=GS))
                        
y_hat <- predict(fit, newdata = true_p, type="prob")
true_p$p <- pmax(loess(y_hat[,2]~ x_1 * x_2, data= true_p, degree=1, span=1/5)$fitted, 0)

## now we create the example dataset
## it includes the labels, the two predictors, and an index so we can recuperate the image from original mnist data
set.seed(1971)
ind <- sample.int(length(index_27), 1000)
labels <- y[ind]
x <- x[ind, ]
index_train <- createDataPartition(labels, p=0.8, list = FALSE)

mnist_27 <- list(
  train = data.frame(y = factor(labels[index_train]),
                     x_1 = x[index_train,1],
                     x_2 = x[index_train,2]),
  test = data.frame(y = factor(labels[-index_train]),
                     x_1 = x[-index_train,1],
                     x_2 = x[-index_train,2]),
  index_train = index_27[ind][index_train],
  index_test = index_27[ind][-index_train],
  true_p = true_p)

save(mnist_27, file = "data/mnist_27.rda", compress="xz")

# check
#ggplot2::ggplot(mnist_27$true_p, aes(x_1, x_2, fill = p)) + geom_raster()+ scale_fill_gradientn(colors=c("#00BFC4","white","#F8766D"))





       

#' Contagious disease data for US states
#' 
#' Yearly counts for Hepatitis A, Measles, Mumps, Pertussis, Polio,  Rubella, and Smallpox for US states. Original data courtesy of Tycho Project (http://www.tycho.pitt.edu/).
#'
#'
#' \itemize{
#'  \item disease. A factor containing disease names.
#'  \item state. A factor containing state names.
#'  \item year. 
#'  \item weeks_reporting. Number of weeks counts were reported that year.
#'  \item count. Total number of reported cases. 
#'  \item population. State population, interpolated for non-census years.
#' }
#
#' @docType data
#'
#' @usage data(us_contagious_diseases)
#'
#' @format An object of class \code{"data.frame"}. 
#'
#' @keywords datasets 
#'
#' @references Willem G. van Panhuis, John Grefenstette, Su Yon Jung, Nian Shong Chok, Anne Cross, Heather Eng, Bruce Y Lee, Vladimir Zadorozhny, Shawn Brown, Derek Cummings, Donald S. Burke. Contagious Diseases in the United States from 1888 to the present. NEJM 2013; 369(22): 2152-2158.
#'
#' @source \href{http://www.tycho.pitt.edu/}{Tycho Project}
#'
#' @examples
#' data(us_contagious_diseases)
#' head(us_contagious_diseases)
"us_contagious_diseases"

#' Greenhouse gas concentrations over 2000 years
#' 
#' Concentrations of the three main greenhouse gases carbon dioxide, methane and nitrous oxide. Measurements are from the Law Dome Ice Core in Antarctica. Selected measurements are provided every 20 years from 1-2000 CE.
#'
#'
#' \itemize{
#'  \item year. Year (CE).
#'  \item gas. Gas being measured: carbon dioxide (`CO2`), methane (`CH4`) or nitrous oxide (`N2O`). 
#'  \item concentration. Gas concentration in ppm by volume (`CO2`) or ppb by volume (`CH4`, `N2O`).
#' }
#'
#' @docType data
#'
#' @usage data(greenhouse_gases)
#'
#' @format An object of class \code{"data.frame"}. 
#'
#' @keywords datasets
#'
#'
#' @source MacFarling Meure et al. 2006 via \href{https://www.ncdc.noaa.gov/paleo-search/study/9959}{NOAA}.
#'
#' @examples
#' data(greenhouse_gases)
#' head(greenhouse_gases)
"greenhouse_gases"

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/murders.R
\docType{data}
\name{murders}
\alias{murders}
\title{US gun murders by state for 2010}
\format{
An object of class \code{"data.frame"}.
}
\source{
\href{https://en.wikipedia.org/wiki/Murder_in_the_United_States_by_state}{Wikipedia}
}
\usage{
data(murders)
}
\description{
Gun murder data from FBI reports. Also contains the population of each state.
}
\details{
\itemize{
 \item state. US state
 \item abb. Abbreviation of US state
 \item region. Geographical US region
 \item population. State population (2010)
 \item total. Number of gun murders in state (2010)
}
}
\examples{
data(murders)
print(murders)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tissue_gene_expression.R
\docType{data}
\name{tissue_gene_expression}
\alias{tissue_gene_expression}
\title{Gene expression profiles for 189 biological samples taken from seven different tissue types.}
\format{
An object of class \code{list}.
}
\source{
\href{https://github.com/genomicsclass/tissuesGeneExpression}{https://github.com/genomicsclass/tissuesGeneExpression}
}
\usage{
data(tissue_gene_expression)
}
\description{
This is a subset of the data provided by the \code{tissuesGeneExpression}
package available from the \code{genomicsclass} GitHub repository. 
The predictors are gene expression measurements from 500 genes that
are a random subset of the original 22,215.
}
\details{
The example dataset is recommended for illustrating
clustering and machine learning techniques.


\itemize{
\item x. The predictors composed of 500 genes. Each row is a gene expression profile and each column is different gene. The column names are the gene symbols.
\item y. The outcomes. A character vector representing the tissue. One of seven tissue types.
}
}
\examples{
data(tissue_gene_expression)
table(tissue_gene_expression$y)
dim(tissue_gene_expression$x)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/brexit_polls.R
\docType{data}
\name{brexit_polls}
\alias{brexit_polls}
\title{Brexit Poll Data}
\format{
An object of class \code{"data.frame"}.
}
\source{
\href{https://en.wikipedia.org/w/index.php?title=Opinion_polling_for_the_United_Kingdom_European_Union_membership_referendum&oldid=896735054}{Wikipedia}
}
\usage{
data(brexit_polls)
}
\description{
Brexit (EU referendum) poll outcomes for 127 polls from January 2016 to the referendum date on June 23, 2016.
}
\details{
\itemize{
 \item startdate. Start date of poll.
 \item enddate. End date of poll.
 \item pollster. Pollster conducting the poll.
 \item poll_type. Online or telephone poll.
 \item samplesize. Sample size of poll.
 \item remain. Proportion voting Remain.
 \item leave. Proportion voting Leave.
 \item undecided. Proportion of undecided voters.
 \item spread. Spread calculated as remain - leave.
}
}
\examples{
data(brexit_polls)
head(brexit_polls)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/take_poll.R
\name{take_poll}
\alias{take_poll}
\title{Models results from taking a poll}
\usage{
take_poll(n, ...)
}
\arguments{
\item{n}{Sample size}

\item{...}{additional arguments to be used by the function \code{sample}.}
}
\value{
None
}
\description{
The function shows a plot of a random sample drawn from an urn with blue and red beads. The sample is taken with replacement. The proportion of blue beads is not shown so that students can try to estimate it.
}
\examples{
take_poll(25)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/outlier_example.R
\docType{data}
\name{outlier_example}
\alias{outlier_example}
\title{Adult male heights in feet with outliers}
\format{
An object of class \code{"numeric"}.
}
\usage{
data(outlier_example)
}
\description{
This dataset was randomly generated with a normal distribution (average: 5 feet 9 inches, standard deviation: 3 inches). One value was changed to be mistakenly reported in centimeters rather than feet.
}
\examples{
data(outlier_example)
mean(outlier_example)
median(outlier_example)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/polls_2008.R
\docType{data}
\name{polls_2008}
\alias{polls_2008}
\title{Poll data for popular vote in 2008 presidential election}
\format{
An object of class \code{data.frame}.
}
\source{
\href{https://web.archive.org/web/20161108190914/http://www.pollster.com/08USPresGEMvO-2.html}{https://web.archive.org/web/20161108190914/http://www.pollster.com/08USPresGEMvO-2.html}
}
\usage{
data(polls_2008)
}
\description{
Data from different pollsters for the popular vote between Obama and McCain 
in the 2008 presidential election.
}
\details{
\itemize{
\item day. Days until election day. Negative numbers are reported so that days can increase up to 0, which is election day.
\item margin. Average difference between Obama and McCain for that day.
}
}
\examples{
data(polls_2008)
with(polls_2008, plot(day, margin))

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gapminder.R
\docType{data}
\name{gapminder}
\alias{gapminder}
\alias{oecd}
\alias{opec}
\title{Gapminder Data}
\format{
An object of class \code{"data.frame"}.
}
\usage{
data(gapminder)
}
\description{
Health and income outcomes for 184 countries from 1960 to 2016. Also includes two character vectors, \code{oecd} 
and \code{opec}, with the names of OECD and OPEC countries from 2016.
}
\details{
\itemize{
 \item country.  
 \item year.
 \item infant_mortality. Infant deaths per 1000.
 \item life_expectancy. Life expectancy in years.
 \item fertility. Average number of children per woman. 
 \item population. Country population.
 \item gpd. GDP according to World Bankdev.
 \item continent. 
 \item region. Geographical region.
}
}
\examples{
data(gapminder)
head(gapminder)
print(oecd)
print(opec)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/results_us_election_2012.R
\docType{data}
\name{results_us_election_2012}
\alias{results_us_election_2012}
\title{2012 US presidential election results}
\format{
An object of class \code{"data.frame"}.
}
\source{
Wikepedia: \url{https://en.wikipedia.org/w/index.php?title=2012_United_States_presidential_election&oldid=1264588444}
}
\usage{
results_us_election_2012
}
\description{
Percentages for the four major candidates by state in the US 2012 presidential elections. It includes congressional districts
for Maine and Nebraska.
}
\details{
\itemize{
\item state. State in which poll was taken. `U.S` is for national polls.
\item electoral_votes. Electoral votes for that state.
\item obama. Percent obtained by Barack Obama.
\item romney. Percent obtained by Mitt Romney.
\item johnson. Percent obtained by Gary Johnson.
\item stein. Percent obtained by Jill Stein.
}
}
\examples{
head(results_us_election_2012)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nyc_regents_scores.R
\docType{data}
\name{nyc_regents_scores}
\alias{nyc_regents_scores}
\title{NYC Regents exams scores 2010}
\format{
An object of class \code{"data.frame"}.
}
\source{
New York City Department of Education via Amanda Cox.
}
\usage{
data(nyc_regents_scores)
}
\description{
Distribution of scores for New York City Regents algebra, global history, biology, English, and U.S. history exams. 
These data were used to make \href{http://graphics8.nytimes.com/images/2011/02/19/nyregion/19schoolsch/19schoolsch-popup.gif}{this New York Times plot}.
}
\details{
\itemize{
\item score. Test score from 0 to 100.
 \item integrated_algebra. Score frequency on Algebra exam.
 \item global_history. Score frequency on global history exam.
 \item living_environment.  Score frequency on biology exam.
 \item english. Score frequency on English exam.
 \item us_history. Score frequency on  U.S. history exam.
}
}
\examples{
data(nyc_regents_scores)
print(nyc_regents_scores)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pr_death_counts.R
\docType{data}
\name{pr_death_counts}
\alias{pr_death_counts}
\title{Puerto Rico daily mortality}
\format{
An object of class \code{data.frame}.
}
\source{
Puerto Rico Demographic Registry. Data was extracted from PDF provided in `system.file("extdata", "RD-Mortality-Report_2015-18-180531.pdf", package = "dslabs")`
}
\usage{
pr_death_counts
}
\description{
A data frame with Puerto Rico daily mortality counts 2015 to May 2018.
This includes the day hurricanes Maria made 2017-09-20.
}
\details{
\itemize{
\item date. Date of the count.
\item deaths. Number of deaths reported that day.
}
}
\examples{

with(pr_death_counts, plot(date, deaths))

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/death_prob.R
\docType{data}
\name{death_prob}
\alias{death_prob}
\title{2015 US Period Life Table}
\format{
An object of class \code{"data.frame"}.
}
\source{
\href{https://www.ssa.gov/oact/STATS/table4c6.html}{Social Security Administraton}
}
\usage{
data(death_prob)
}
\description{
Probability of death within 1 year by age and sex in the United States in 2015.
}
\details{
\itemize{
 \item age. Age strata, with each year a different stratum.
 \item sex. Male or Female.
 \item prob. Probability of death within 1 year given exact age and sex.
}
}
\examples{
data(death_prob)
head(death_prob)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/na_example.R
\docType{data}
\name{na_example}
\alias{na_example}
\title{Count data with some missing values}
\format{
An object of class \code{"integer"}.
}
\usage{
data(na_example)
}
\description{
This dataset was randomly generated.
}
\examples{
data(na_example)
print(sum(is.na(na_example)))
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_mnist.R
\name{read_mnist}
\alias{read_mnist}
\title{Download and read the mnist dataset}
\usage{
read_mnist(
  path = NULL,
  download = FALSE,
  destdir = tempdir(),
  url = "https://www2.harvardx.harvard.edu/courses/IDS_08_v2_03/",
  keep.files = TRUE
)
}
\arguments{
\item{path}{A character giving the full path of the directory to look for files. It assumes the filenames are the same as the originals. If path is \code{NULL} a download or direct read of the files is attempted.}

\item{download}{If \code{TRUE} the files will be downloaded and saved in detsdir.}

\item{destdir}{A character giving the full path of the directory in which to save the downloaded files. The default is to use a temporary directory.}

\item{url}{A character giving the URL from which to download files. Currently a copy of the data is available at https://www2.harvardx.harvard.edu/courses/IDS_08_v2_03/, the current default URL.}

\item{keep.files}{A logical. If \code{TRUE} the downloaded files will be saved in \code{destdir}. If \code{FALSE} the entire directory is erased. This argument is ignored if \code{download} is \code{FALSE}.){
}}
}
\value{
A list with two components: train and test. Each of these is a list with two components: images and labels. The images component is a
matrix with each column representing one of the 28*28 = 784 pixels. The values are integers between 0 and 255 representing grey scale. 
The labels components is a vector representing the digit shown in the image.

Note that the data is over 10MB, so the download may take several seconds depending on internet speed. If you plan to load the data more than once
we recommend you download the data once and read it from disk in the future. See examples.
}
\description{
This function downloads the mnist training and test data available here http://yann.lecun.com/exdb/mnist/
}
\examples{
# this can take several seconds, depending on internet speed.

\dontrun{
mnist <- read_mnist()
i <- 5
image(1:28, 1:28, matrix(mnist$test$images[i,], nrow=28)[ , 28:1], 
    col = gray(seq(0, 1, 0.05)), xlab = "", ylab="")
## the labels for this image is: 
mnist$test$labels[i]
}

# You can download and save the data to a directory like this:
\dontrun{
mnist <- read_mnist(download = TRUE, destdir = "~/Downloads")

# and then, going forward, read from disk 
mnist <- read_mnist("~/Downloads")
}
}
\references{
Y. LeCun, L. Bottou, Y. Bengio, and P. Haffner. "Gradient-based learning applied to document recognition." Proceedings of the IEEE, 86(11):2278-2324, November 1998.
}
\author{
Samuela Pollack

Rafael A. Irizarry, \email{rafael_irizarry@dfci.harvard.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/temp_carbon.R
\docType{data}
\name{temp_carbon}
\alias{temp_carbon}
\title{Global temperature anomaly and carbon emissions, 1751-2018}
\format{
An object of class \code{"data.frame"}.
}
\source{
\href{https://www.ncdc.noaa.gov/cag/global/time-series}{NOAA} and Boden, T.A., G. Marland, and R.J. Andres (2017) via \href{https://cdiac.ess-dive.lbl.gov/trends/emis/tre_glob_2014.html}{CDIAC}
}
\usage{
data(temp_carbon)
}
\description{
Annual mean global temperature anomaly on land, sea and combined, 1880-2018. Annual global carbon emissions, 1751-2014.
}
\details{
\itemize{
 \item year. Year (CE).
 \item temp_anomaly. Global annual mean temperature anomaly in degrees Celsius relative to the 20th century mean temperature. 1880-2018. 
 \item land_anomaly. Annual mean temperature anomaly on land in degrees Celsius relative to the 20th century mean temperature. 1880-2018.
 \item ocean_anomaly. Annual mean temperature anomaly over ocean in degrees Celsius relative to the 20th century mean temperature. 1880-2018.
 \item carbon_emissions. Annual carbon emissions in millions of metric tons of carbon. 1751-2014.
}
}
\examples{
data(temp_carbon)
head(temp_carbon)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/olive.R
\docType{data}
\name{olive}
\alias{olive}
\title{Italian olive}
\format{
An object of class \code{"data.frame"}.
}
\source{
J. Zupan, and J. Gasteiger.  Neural Networks in Chemistry and Drug Design.
}
\usage{
data(olive)
}
\description{
Composition in percentage of eight fatty acids found in the lipid fraction of 572 Italian olive oils
}
\details{
\itemize{
\item region. General region of Italy.
\item area. Area of Italy. 
\item palmitic. Percent palmitic acid of sample. 
\item palmitoleic. Percent palmitoleic of sample. 
\item stearic. Percent stearic acid of sample.  
\item oleic. Percent oleic acid of sample.  
\item linoleic. Percent linoleic acid of sample.  
\item linolenic. Percent linolenic acid of sample.  
\item arachidic. Percent arachidic acid of sample.  
\item eicosenoic. Percent eicosenoic acid of sample.  
}
}
\examples{
data(olive)
head(olive)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mice_weights.R
\docType{data}
\name{mice_weights}
\alias{mice_weights}
\title{Mice weights}
\format{
An object of class \code{"data.frame"}.
}
\source{
Karen Svenson, Daniel M. Gatti, and Gary Churchill from Jackson Laboratories.
}
\usage{
mice_weights
}
\description{
Body weights, bone density, and percent fat for mice under two diets: chow and high fat. Data provided by Karen Svenson from Jackson Laboratories. Funding to generate these data came from NIH grant P50 GM070683 awarded to Gary Churchill.
}
\details{
\itemize{
  \item body_weight. Body weight in grams at 19 weeks.
  \item bone_density. Body density.
  \item percent_fat. Percent fat.
  \item sex. The sex of the mice.
  \item diet. The diet of the mice: chow or high fat.
  \item gen. These are outbread mice. This variable denotes the generation.
  \item litter. Which of two litters mice belong to.
}
}
\examples{
mice_weights |> head()
with(mice_weights, table(sex, diet))
}
\references{
Daniel M. Gatti, Petr Simecek, Lisa Somes, Clifton T. Jeffrey, Matthew J. Vincent, Kwangbom Choi, Xingyao Chen, Gary A. Churchill, and Karen L. Svenson. "The Effects of Sex and Diet on Physiology and Liver Gene Expression in Diversity Outbred Mice". bioRxiv 098657; \doi{10.1101/098657}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/us_contagious_disease.R
\docType{data}
\name{us_contagious_diseases}
\alias{us_contagious_diseases}
\title{Contagious disease data for US states}
\format{
An object of class \code{"data.frame"}.
}
\source{
\href{http://www.tycho.pitt.edu/}{Tycho Project}
}
\usage{
data(us_contagious_diseases)
}
\description{
Yearly counts for Hepatitis A, Measles, Mumps, Pertussis, Polio,  Rubella, and Smallpox for US states. Original data courtesy of Tycho Project (http://www.tycho.pitt.edu/).
}
\details{
\itemize{
 \item disease. A factor containing disease names.
 \item state. A factor containing state names.
 \item year. 
 \item weeks_reporting. Number of weeks counts were reported that year.
 \item count. Total number of reported cases. 
 \item population. State population, interpolated for non-census years.
}
}
\examples{
data(us_contagious_diseases)
head(us_contagious_diseases)
}
\references{
Willem G. van Panhuis, John Grefenstette, Su Yon Jung, Nian Shong Chok, Anne Cross, Heather Eng, Bruce Y Lee, Vladimir Zadorozhny, Shawn Brown, Derek Cummings, Donald S. Burke. Contagious Diseases in the United States from 1888 to the present. NEJM 2013; 369(22): 2152-2158.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mnist_27.R
\docType{data}
\name{mnist_27}
\alias{mnist_27}
\title{Useful example for illustrating machine learning algorithms based on MNIST data}
\format{
An object of class \code{list}.
}
\usage{
data(mnist_27)
}
\description{
We only include a randomly selected set of 2s and 7s along with the two predictors based on the 
proportion of dark pixels in the upper left and lower right quadrants respectively. The dataset is divided into 
training and test sets.
}
\details{
\itemize{
\item train. A data frame containing  training data: labels and predictors.
\item test. A data frame containing  test data: labels and predictors.
\item index_train. The index of the original mnist training data used for the training set.
\item index_test. The index of the original mnist test data used for the test set.
\item true_p. A \code{data.frame} containing the 
two predictors \code{x_1} and \code{x_2} and the conditional probability of being a 7 
for \code{x_1}, \code{x_2}.
}
}
\examples{
data(mnist_27)
with(mnist_27$train, plot(x_1, x_2, col = as.numeric(y)))

}
\references{
Y. LeCun, L. Bottou, Y. Bengio, and P. Haffner. "Gradient-based learning applied to document recognition." Proceedings of the IEEE, 86(11):2278-2324, November 1998.
}
\seealso{
[read_mnist()]
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/admissions.R
\docType{data}
\name{admissions}
\alias{admissions}
\title{Gender bias among graduate school admissions to UC Berkeley.}
\format{
An object of class \code{"data.frame"}.
}
\source{
PJ Bickel, EA Hammel, and JW O'Connell. Science (1975)
}
\usage{
data(admissions)
}
\description{
The admission data for six majors for the fall of 1973; often used as an example of Simpson's paradox
}
\details{
\itemize{
  \item major. The major or university department.
  \item gender. Men or women.
  \item admitted. Percent of students admitted.
  \item applicants. Total number of applicants.
}
}
\examples{
data(admissions)
admissions
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heights.R
\docType{data}
\name{heights}
\alias{heights}
\title{Self-reported Heights in Inches}
\format{
An object of class \code{"data.frame"}.
}
\usage{
heights
}
\description{
Self-reported heights and sex. The heights were converted to inches from the original data included in \code{\link{reported_heights}}.
}
\details{
\itemize{
 \item sex. A factor with the self-reported sex. 
 \item height. A numeric vector with self-reported heights in inches. 
}
}
\examples{
head(heights)
}
\seealso{
\code{\link{reported_heights}} for the original data source.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reported_heights.R
\docType{data}
\name{reported_heights}
\alias{reported_heights}
\title{Self-reported Heights}
\format{
An object of class \code{"data.frame"}.
}
\usage{
data(reported_heights)
}
\description{
Students were asked to report their height (in inches) and sex in an online form. 
This table includes the results from four courses.
}
\details{
\itemize{
 \item time_stamp. Time and date of the entry.
 \item sex. Sex of the student.
 \item height. Height as reported by student by filling in a text free box. 
}
}
\examples{
data(reported_heights)
head(reported_heights)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/greenhouse_gases.R
\docType{data}
\name{greenhouse_gases}
\alias{greenhouse_gases}
\title{Greenhouse gas concentrations over 2000 years}
\format{
An object of class \code{"data.frame"}.
}
\source{
MacFarling Meure et al. 2006 via \href{https://www.ncdc.noaa.gov/paleo-search/study/9959}{NOAA}.
}
\usage{
data(greenhouse_gases)
}
\description{
Concentrations of the three main greenhouse gases carbon dioxide, methane and nitrous oxide. Measurements are from the Law Dome Ice Core in Antarctica. Selected measurements are provided every 20 years from 1-2000 CE.
}
\details{
\itemize{
 \item year. Year (CE).
 \item gas. Gas being measured: carbon dioxide (`CO2`), methane (`CH4`) or nitrous oxide (`N2O`). 
 \item concentration. Gas concentration in ppm by volume (`CO2`) or ppb by volume (`CH4`, `N2O`).
}
}
\examples{
data(greenhouse_gases)
head(greenhouse_gases)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ds_theme_set.R
\name{ds_theme_set}
\alias{ds_theme_set}
\title{dslabs theme set}
\usage{
ds_theme_set(
  new = "theme_bw",
  args = NULL,
  base_size = 11,
  bold_title = TRUE,
  ...
)
}
\arguments{
\item{new}{a prebuilt ggplot2 theme. Defaults to \code{"theme_minimal"}}

\item{args}{the arguments to be passed along to the ggplot2 theme function. Defaults to \code{"NULL"}.}

\item{base_size}{if \code{"args"} is \code{"NULL"}, base_size is one of the arguments passed to the theme function. It defaults to 11.}

\item{bold_title}{if TRUE, sets titles to be bold}

\item{...}{additional arguments to be used by theme}
}
\value{
None
}
\description{
This function sets a ggplot2 theme used throughout the data science labs. It can be called without arguments.
}
\examples{
library(ggplot2)
ds_theme_set()
qplot(hp, mpg, data=mtcars, color=am, facets=gear~cyl, 
main="Scatterplots of MPG vs. Horsepower", 
xlab="Horsepower", ylab="Miles per Gallon")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/trump_tweets.R
\docType{data}
\name{trump_tweets}
\alias{trump_tweets}
\title{Trump Tweets from2009 to 2017}
\format{
An object of class \code{"data.frame"}.
}
\source{
The Trump Twitter Archive: \url{https://www.thetrumparchive.com/}
}
\usage{
data(trump_tweets)
}
\description{
All tweets from Donald Trump's twitter account from 2009 to 2017
}
\details{
\itemize{
\item source. Device or service used to compose tweet.
\item id_str. Tweet ID.
\item text. Tweet.
\item created_at. Data and time tweet was tweeted.
\item retweet_count. How many times tweet had been retweeted at time dataset was created.
\item in_reply_to_user_id_str. If a reply, the user id of person being replied to.
\item favorite_count. Number of times tweet had been favored at time dataset was created.
\item is_retweet. A logical telling us if it is a retweet or not.
}
}
\examples{
data(trump_tweets)
head(trump_tweets)

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/brca.R
\docType{data}
\name{brca}
\alias{brca}
\title{Breast Cancer Wisconsin Diagnostic Dataset from UCI Machine Learning Repository}
\format{
An object of class \code{list}.
}
\source{
\href{https://archive.ics.uci.edu/ml/datasets/}{UCI Machine Learning Repository}
}
\usage{
data(brca)
}
\description{
Biopsy features for classification of 569 malignant (cancer) and benign (not cancer) breast masses.
}
\details{
Features were computationally extracted from digital images of fine needle aspirate
biopsy slides. Features correspond to properties of cell nuclei, such as size, shape and regularity.
The mean, standard error, and worst value of each of 10 nuclear parameters is reported for a total of 30 features.

This is a classic dataset for training and benchmarking machine learning algorithms.


\itemize{
\item y. The outcomes. A factor with two levels denoting whether a mass is malignant ("M") or benign ("B").
\item x. The predictors. A matrix with the mean, standard error and worst value of each of 10 nuclear measurements on the slide, for 30 total features per biopsy:
    \itemize{
      \item radius. Nucleus radius (mean of distances from center to points on perimeter).
      \item texture. Nucleus texture (standard deviation of grayscale values).
      \item perimeter. Nucleus perimeter.
      \item area. Nucleus area.
      \item smoothness. Nucleus smoothness (local variation in radius lengths).
      \item compactness. Nucleus compactness (perimeter^2/area - 1).
      \item concavity, Nucleus concavity (severity of concave portions of the contour).
      \item concave_pts. Number of concave portions of the nucleus contour.
      \item symmetry. Nucleus symmetry.
      \item fractal_dim. Nucleus fractal dimension ("coastline approximation" -1).
    }
}
}
\examples{
data(brca)
table(brca$y)
dim(brca$x)
head(brca$x)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/historic_co2.R
\docType{data}
\name{historic_co2}
\alias{historic_co2}
\title{Atmospheric carbon dioxide concentration over 800,000 years}
\format{
An object of class \code{"data.frame"}.
}
\source{
Mauna Loa data from \href{https://www.esrl.noaa.gov/gmd/ccgg/trends/data.html}{NOAA}. Ice core data from Bereiter et al. 2015 via \href{https://www.ncdc.noaa.gov/paleo-search/study/17975}{NOAA}.
}
\usage{
data(historic_co2)
}
\description{
Concentration of carbon dioxide in ppm by volume from direct measurements at Mauna Loa (1959-2018 CE) and indirect measurements from a series of Antarctic ice cores (approx. -800,000-2001 CE).
}
\details{
\itemize{
 \item year. Year (CE).
 \item co2. Carbon dioxide concentration in ppm by volume.
 \item source. Source of carbon dioxide measurement: direct CO2 annual mean concentrations from Mauna Loa (`Mauna Loa`) or indirect CO2 concentrations from air trapped in ice cores (`Ice Cores`).
}
}
\examples{
data(historic_co2)
head(historic_co2)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stars.R
\docType{data}
\name{stars}
\alias{stars}
\title{Physical Properties of Stars}
\format{
An object of class \code{"data.frame"}.
}
\source{
Compiled from multiple open-access references on \href{http://vizier.u-strasbg.fr/viz-bin/VizieR}{VizieR}.
}
\usage{
data(stars)
}
\description{
Physical properties of selected stars, including luminosity, temperature, and spectral class.
}
\details{
\itemize{
 \item star. Name of star.
 \item magnitude. Absolute magnitude of the star, which is a function of the star's luminosity and distance to the star.
 \item temp. Surface temperature in degrees Kelvin (K).
 \item type. Spectral class of star in the OBAFGKM system.
}
}
\examples{
data(stars)
head(stars)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/divorce_margarine.R
\docType{data}
\name{divorce_margarine}
\alias{divorce_margarine}
\title{Divorce rate and margarine consumption data}
\format{
An object of class \code{"data.frame"}.
}
\source{
\href{http://www.tylervigen.com/spurious-correlations}{Spurious Correlations}
}
\usage{
data(divorce_margarine)
}
\description{
Divorce rates in Maine and per capita consumption of margarine in US data
}
\details{
\itemize{
  \item divorce_rate_maine. Divorce per 1000 in Maine.
  \item margarine_consumption_per_capita. US per capita consumption of margarine in pounds.
  \item year. Year.
}
}
\examples{
data(divorce_margarine)
with(divorce_margarine, plot(margarine_consumption_per_capita, divorce_rate_maine))
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_recommender_model.R
\name{fit_recommender_model}
\alias{fit_recommender_model}
\title{Fit a Latent Factor Recommender Model via Alternating Least Squares}
\usage{
fit_recommender_model(
  rating,
  user_id,
  item_id,
  K = 8,
  lambda_1 = 5e-05,
  lambda_2 = 1e-04,
  min_ratings = 20,
  maxit = 500,
  reltol = 1e-08,
  damping = 0.75,
  verbose = FALSE
)
}
\arguments{
\item{rating}{A numeric vector of observed ratings.}

\item{user_id}{A character vector identifying the user
for each rating. Must be the same length as `rating`.}

\item{item_id}{A character vector identifying the item
for each rating. Must be the same length as `rating`.}

\item{K}{Integer. The number of latent factors to estimate.}

\item{lambda_1}{Numeric. Regularization parameter for user and item effects.}

\item{lambda_2}{Numeric. Regularization parameter for latent factors.}

\item{min_ratings}{Integer. Minimum number of ratings required for an item to
be included in the estimation of latent factors.}

\item{maxit}{Integer. Maximum number of iterations.}

\item{reltol}{Numeric. Relative reltolerance for convergence, based on change in
the objective function.}

\item{damping}{Numeric between 0 and 1. Damping factor used to blend updates
with the previous iteration for convergence stability.}

\item{verbose}{Logical. If `TRUE`, prints progress messages during
optimization.}
}
\value{
A list with the following components:
  \item{mu}{Global mean rating.}
  \item{a}{Named numeric vector of user-specific effects.}
  \item{b}{Named numeric vector of item-specific effects.}
  \item{p}{Matrix of user latent factors, with one row per user. The rownames of this matrix match the names of `a`.}
  \item{q}{Matrix of item latent factors, with one row per item. The rownames of this matrix match the names of `b`.}
  \item{min_ratings}{The threshold value used to filter items: only items with at least this many ratings are included in the estimation of latent factors.}
  \item{n_item}{Named integer vector of number of ratings per item.}
  \item{n_user}{Named integer vector of number of retained ratings per user.}
  \item{fitted}{Fitted values.}
}
\description{
This function fits a penalized latent factor model for recommendation systems
using an alternating least squares (ALS) algorithm. It estimates user effects,
item effects, and latent factors simultaneously, with regularization to
prevent overfitting. The implementation supports filtering out items with too
few ratings.
}
\details{
The model being fit is:
\deqn{Y_{ij} = \mu + \alpha_i + \beta_j + \sum_{k=1}^K p_{ik} q_{jk} + \varepsilon_{ij}}
where \eqn{\mu} is the global mean, \eqn{\alpha_i} are user effects,
\eqn{\beta_j} are item effects, and the \eqn{p_{ik}}, \eqn{q_{jk}} are latent
factors for users and items respectively. The estimation minimizes a penalized
least squares criterion with separate penalties for user/item effects and
latent factors.

Items with less than min_ratings observations are excluded from the estimation 
of `p` and `q`.
}
\examples{
set.seed(2010)
## Simulation settings
n_users <- 200       # number of users
n_items <- 300       # number of items
K_true  <- 4         # true number of latent factors
sparsity <- 0.25     # ~5\% of user-item pairs are observed

## True parameters
mu <- 3.5
a_true <- rnorm(n_users, 0, 0.3)     # user effects
b_true <- rnorm(n_items, 0, 0.4)     # item effects
p_true <- matrix(rnorm(n_users * K_true, 0, 0.5), n_users, K_true)
q_true <- matrix(rnorm(n_items * K_true, 0, 0.5), n_items, K_true)

names(a_true) <- 1:n_users
names(b_true) <- 1:n_items
rownames(p_true) <- 1:n_users
rownames(q_true) <- 1:n_items
## Generate observed ratings matrix with sparsity
user_id <- rep(as.character(1:n_users), each = n_items)
item_id <- rep(as.character(1:n_items), times = n_users)


## Which entries are observed?
obs <- runif(length(user_id)) < sparsity

## Ratings with noise
rating_full <- mu + a_true[user_id] + b_true[item_id] +
  rowSums(p_true[user_id, ] * q_true[item_id, ]) +
  rnorm(length(user_id), 0, 0.25)

rating <- rating_full[obs]
user_id <- user_id[obs]
item_id <- item_id[obs]

## Call your recommender function
fit <- fit_recommender_model(rating, user_id, item_id, K = 4, reltol = 1e-5,
                             min_ratings = 5, verbose = TRUE)
plot(fit$fitted, rating)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/polls_us_election_2016.R
\docType{data}
\name{polls_us_election_2016}
\alias{polls_us_election_2016}
\alias{results_us_election_2016}
\title{Fivethirtyeight 2016 Poll Data}
\format{
An object of class \code{"data.frame"}.
}
\source{
The original csv file used to create \code{polls_us_election_2016} is here: \url{https://projects.fivethirtyeight.com/general-model/president_general_polls_2016.csv} 

The data for \code{results_us_election_2016} is from Ballotpedia and can be found here:
\url{https://docs.google.com/spreadsheets/d/1zxyOQDjNOJS_UkzerorUCf2OAdcMcIQEwRciKuYBIZ4/pubhtml?widget=true&headers=false#gid=658726802/}
}
\usage{
data(polls_us_election_2016)
}
\description{
Poll results from US 2016 presidential elections aggregated from HuffPost Pollster, RealClearPolitics, polling firms, and news reports. 
The dataset also includes election results (popular vote) and electoral college votes in \code{results_us_election_2016}.
}
\details{
\itemize{
\item state. State in which poll was taken. `U.S` is for national polls.
\item startdate. Poll's start date.
\item enddate. Poll's end date.
\item pollster. Pollster conducting the poll.
\item grade. Grade assigned by fivethirtyeight to pollster.
\item samplesize. Sample size. 
\item population. Type of population being polled.
\item rawpoll_clinton. Percentage for Hillary Clinton.
\item rawpoll_trump. Percentage for Donald Trump
\item rawpoll_johnson. Percentage for Gary Johnson
\item rawpoll_mcmullin. Percentage for Evan McMullin.
\item adjpoll_clinton. Fivethirtyeight adjusted percentage for Hillary Clinton.
\item ajdpoll_trump. Fivethirtyeight adjusted percentage for Donald Trump
\item adjpoll_johnson. Fivethirtyeight adjusted percentage for Gary Johnson
\item adjpoll_mcmullin. Fivethirtyeight adjusted percentage for Evan McMullin.
}
}
\examples{
data(polls_us_election_2016)
head(polls_us_election_2016)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/movielens.R
\docType{data}
\name{movielens}
\alias{movielens}
\title{Movie ratings}
\format{
Two object of class \code{data.frame}.
}
\source{
\href{http://files.grouplens.org/datasets/movielens/ml-latest-small.zip}{http://files.grouplens.org/datasets/movielens/ml-latest-small.zip}
}
\usage{
data(movielens)
}
\description{
MovieLens Latest Dataset (Small)
}
\details{
\itemize{
\item movieId. Unique ID for the movie.
\item title. Movie title (not unique). 
\item year. Year the movie was released.
\item genres. Genres associated with the movie.
\item userId. Unique ID for the user. 
\item rating. A rating between 0 and 5 for the movie.
\item timestamp. Date and time the rating was given.
}
}
\examples{
data(movielens)
head(movielens)
}
\references{
F. Maxwell Harper and Joseph A. Konstan. 2015. The MovieLens Datasets: History and Context. ACM Transactions on Interactive Intelligent Systems (TiiS) 5, 4, Article 19 (December 2015), 19 pages. DOI=http://dx.doi.org/10.1145/2827872
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/research_funding_rates.R
\docType{data}
\name{research_funding_rates}
\alias{research_funding_rates}
\alias{raw_data_research_funding_rates}
\title{Gender bias in research funding in the Netherlands}
\format{
An object of class \code{"data.frame"}.
}
\source{
van der Lee and Ellemers (2015) PNAS \url{https://www.pnas.org/content/112/40/12349.abstract/}
}
\usage{
data(research_funding_rates)
}
\description{
Table S1 from paper title "Gender contributes to personal research funding success in The Netherlands"
}
\details{
\itemize{
  \item discipline. Research area discipline.
  \item applications_total. Total applications.
  \item applications_men. Total applications by men.
  \item applications_women. Total applications by women.
  \item awards_total. Total awards.
  \item awards_men. Total awards received by men.
  \item awards_women. Total awards received by women.
  \item success_rates_total. Overall success rate.
  \item success_rates_men. Success rate for men.
  \item success_rates_women. Success rate for women.
}
}
\examples{
data(research_funding_rates)
research_funding_rates
# The raw data for this table is available from
data(raw_data_research_funding_rates)

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rfalling_object.R
\name{rfalling_object}
\alias{rfalling_object}
\title{Simulate falling object data}
\usage{
rfalling_object(
  n = 14,
  d_0 = 55.86,
  v_0 = 0,
  g = -9.8,
  scale = 1,
  time = seq(0, 3.25, length.out = n),
  error_distribution = c("rnorm", "rt"),
  df = 3
)
}
\arguments{
\item{n}{Sample size}

\item{d_0}{Height from which object will fall in meters.}

\item{v_0}{Initial velocity with which object will fall in meters per second.}

\item{g}{Gravitational constant, 9.8 meters per second per seonnd}

\item{scale}{The measurement errors will be multiplied by this constant.}

\item{time}{Numeric vector of times, in seconds, at which measurements were taken.}

\item{error_distribution}{Character. Either \code{rnorm} for normal or \code{rt} for t-distribution.}

\item{df}{If using t-distribution, the degrees of freedom.}
}
\value{
A \code{data.frame} with the time, the distance travelled, and the observed distance.
}
\description{
The function simulates a falling object's position. Default parameters are for dropping a 
weight from the tower of Pisa.
}
\examples{

dat <- rfalling_object()
with(dat, plot(time, observed_distance))
with(dat, lines(time, distance, col = "blue"))

}
